#!/usr/bin/env python

import json
import os
import re
import sys
from datetime import datetime
from itertools import chain
from collections import OrderedDict
from insights import dr, get_filters
from insights.core.spec_factory import RegistryPoint
from insights.specs import Specs
from insights.core import filters

if len(sys.argv) < 3:
    print("Provide uploader.json location and packages to load")
    sys.exit(1)

json_path = sys.argv[1]

if not os.path.exists(json_path):
    print("Provided uploader.json path does not exist.")
    sys.exit(1)

with open(json_path) as fp:
    uploader_json = json.load(fp, object_pairs_hook=OrderedDict)

dr.load_components("insights.specs.default")
dr.load_components("insights.parsers")
dr.load_components("insights.combiners")


for package in sys.argv[2:]:
    dr.load_components(package)

filters.dump()
specs = sorted(vars(Specs))
filters = {}
for spec in specs:
    s = getattr(Specs, spec)
    if type(s) == RegistryPoint:
        f = get_filters(s)
        if f:
            filters[spec] = sorted(f)

for spec in chain.from_iterable(uploader_json[i] for i in ("commands", "files", "globs")):
    if spec["symbolic_name"] in filters:
        spec["pattern"] = filters[spec["symbolic_name"]]

uploader_json["version"] = datetime.now().isoformat()

pattern = re.compile(", $")
output = "\n".join(pattern.sub(",", l) for l in json.dumps(uploader_json, indent=4).splitlines())

with open(json_path, "w") as fp:
    fp.write(output)



import pytest
import doctest
from insights.parsers import ParseException, gluster_vol
from insights.tests import context_wrap
from insights.parsers.gluster_vol import GlusterVolInfo, GlusterVolStatus


# ---------------------------Gluster Vol Info Tests-----------------------------
TRACKING_VALID = """

Volume Name: test_vol
Type: Replicate
cluster.choose-local: off
network.remote-dio: enable
performance.low-prio-threads: 32
performance.io-cache: off
performance.read-ahead: off
performance.quick-read: off
nfs.disable: on
performance.client-io-threads: off
"""

TRACKING_INVALID = """ """

MULTIPLE_VOLUMES = """
Volume Name: test_vol
Type: Replicate
Volume ID: 2c32ed8d-5a07-4a76-a73a-123859556974
Status: Started
Snapshot Count: 0
Number of Bricks: 1 x 3 = 3
Transport-type: tcp
Bricks:
Brick1: 172.17.18.42:/home/brick
Brick2: 172.17.18.43:/home/brick
Brick3: 172.17.18.44:/home/brick
Options Reconfigured:
cluster.choose-local: off
user.cifs: off
features.shard: on
cluster.shd-wait-qlength: 10000
cluster.shd-max-threads: 8
cluster.locking-scheme: granular
cluster.data-self-heal-algorithm: full
cluster.server-quorum-type: server
cluster.quorum-type: auto
cluster.eager-lock: enable
network.remote-dio: enable
performance.low-prio-threads: 32
performance.io-cache: off
performance.read-ahead: off
performance.quick-read: off
transport.address-family: inet
nfs.disable: on
performance.client-io-threads: off

Volume Name: test_vol_2
Type: Replicate
Volume ID: dd821df9-ee2e-429c-98a0-81b1b794433d
Status: Started
Snapshot Count: 0
Number of Bricks: 1 x 3 = 3
Transport-type: tcp
Bricks:
Brick1: 172.17.18.42:/home/brick2
Brick2: 172.17.18.43:/home/brick2
Brick3: 172.17.18.44:/home/brick2
Options Reconfigured:
cluster.choose-local: off
user.cifs: off
features.shard: on
cluster.shd-wait-qlength: 10000
cluster.shd-max-threads: 8
cluster.locking-scheme: granular
cluster.data-self-heal-algorithm: full
cluster.server-quorum-type: server
cluster.quorum-type: auto
cluster.eager-lock: enable
network.remote-dio: enable
performance.low-prio-threads: 32
performance.io-cache: off
performance.read-ahead: off
performance.quick-read: off
transport.address-family: inet
nfs.disable: on
performance.client-io-threads: off
""".strip()


def test_gluster_vol_info_invalid():
    with pytest.raises(ParseException) as e:
        GlusterVolInfo(context_wrap(TRACKING_INVALID))
    assert "Unable to parse gluster volume options: []" in str(e)


def test_gluster_volume_options():
    parser_result = GlusterVolInfo(context_wrap(TRACKING_VALID))
    assert parser_result is not None

    data = parser_result.data["test_vol"]
    assert data['network.remote-dio'] == 'enable'
    assert data['cluster.choose-local'] == 'off'
    assert data['performance.client-io-threads'] == 'off'
    assert data['performance.quick-read'] == 'off'
    assert data['performance.low-prio-threads'] == '32'
    assert data['performance.io-cache'] == 'off'
    assert data['performance.read-ahead'] == 'off'
    assert data['nfs.disable'] == 'on'


def test_gluster_multiple_volume_options():
    parser_result = GlusterVolInfo(context_wrap(MULTIPLE_VOLUMES))
    assert parser_result is not None

    data = parser_result.data["test_vol"]
    assert data['network.remote-dio'] == 'enable'
    assert data['cluster.choose-local'] == 'off'
    assert data['performance.client-io-threads'] == 'off'
    assert data['performance.quick-read'] == 'off'
    assert data['performance.low-prio-threads'] == '32'
    assert data['performance.io-cache'] == 'off'
    assert data['performance.read-ahead'] == 'off'
    assert data['nfs.disable'] == 'on'

    data = parser_result.data["test_vol_2"]
    assert data['network.remote-dio'] == 'enable'
    assert data['cluster.choose-local'] == 'off'
    assert data['performance.client-io-threads'] == 'off'
    assert data['performance.quick-read'] == 'off'
    assert data['performance.low-prio-threads'] == '32'
    assert data['performance.io-cache'] == 'off'
    assert data['performance.read-ahead'] == 'off'
    assert data['nfs.disable'] == 'on'
# --------------------------------------END------------------------------------


# ---------------------------Gluster Vol Status Tests-----------------------------
VOL_STATUS_BAD = """
""".strip()

VOL_STATUS_GOOD = """
Status of volume: test_vol
Gluster process                             TCP Port  RDMA Port  Online  Pid
------------------------------------------------------------------------------
Brick 172.17.18.42:/home/brick              49152     0          Y       26685
Brick 172.17.18.43:/home/brick              49152     0          Y       27094
Brick 172.17.18.44:/home/brick              49152     0          Y       27060
Self-heal Daemon on localhost               N/A       N/A        Y       7805
Self-heal Daemon on 172.17.18.44            N/A       N/A        Y       33400
Self-heal Daemon on 172.17.18.43            N/A       N/A        Y       33680

Task Status of Volume test_vol
------------------------------------------------------------------------------
There are no active volume tasks
""".strip()

VOL_STATUS_MULTIPLE_GOOD = """
Status of volume: test_vol
Gluster process                             TCP Port  RDMA Port  Online  Pid
------------------------------------------------------------------------------
Brick 172.17.18.42:/home/brick              49152     0          Y       26685
Brick 172.17.18.43:/home/brick              49152     0          Y       27094
Brick 172.17.18.44:/home/brick              49152     0          Y       27060
Self-heal Daemon on localhost               N/A       N/A        Y       7805
Self-heal Daemon on 172.17.18.44            N/A       N/A        Y       33400
Self-heal Daemon on 172.17.18.43            N/A       N/A        Y       33680

Task Status of Volume test_vol
------------------------------------------------------------------------------
There are no active volume tasks


Status of volume: test_vol_2
Gluster process                             TCP Port  RDMA Port  Online  Pid
------------------------------------------------------------------------------
Brick 172.17.18.42:/home/brick              49152     0          Y       6685
Brick 172.17.18.43:/home/brick              49152     0          Y       33094
Brick 172.17.18.44:/home/brick              49152     0          Y       2060
Self-heal Daemon on localhost               N/A       N/A        Y       7805
Self-heal Daemon on 172.17.18.44            N/A       N/A        Y       33400
Self-heal Daemon on 172.17.18.43            N/A       N/A        Y       33680

Task Status of Volume test_vol
------------------------------------------------------------------------------
There are no active volume tasks

""".strip()

MULTIPLE_VOL_STATUS_LONG_HOSTNAME = """
Locking failed on srvgluspvlsu72.example.com. Please check log file for details.
Another transaction is in progress for volQXCHISTP_data. Please try again after sometime.

Status of volume: volQXCHISTPI_log
Gluster process                             TCP Port  RDMA Port  Online  Pid
------------------------------------------------------------------------------
Brick srvgluspvlsu71.example.com:/gluster/Q
XCHISTPI/log/brick                          49153     0          Y       11020
Brick srvgluspvlsu72.example.com:/gluster/Q
XCHISTPI/log/brick                          49153     0          Y       11182
Brick srvgluspvlsu73.example.com:/gluster/Q
XCHISTPI/log/brick                          49153     0          Y       129143
Self-heal Daemon on localhost               N/A       N/A        Y       130186
Self-heal Daemon on srvgluspvlsu71          N/A       N/A        Y       20009
Self-heal Daemon on srvgluspvlsu72.example.
com                                         N/A       N/A        Y       12300

Task Status of Volume volQXCHISTPI_log
------------------------------------------------------------------------------
There are no active volume tasks

Status of volume: volQXSCCINTP_data
Gluster process                             TCP Port  RDMA Port  Online  Pid
------------------------------------------------------------------------------
Brick srvgluspvlsu71.example.com:/gluster/Q
XSCCINTP/data/brick                         49190     0          Y       11374
Brick srvgluspvlsu72.example.com:/gluster/Q
XSCCINTP/data/brick                         49190     0          Y       11524
Brick srvgluspvlsu73.example.com:/gluster/Q
XSCCINTP/data/bri                           49190     0          Y       129476
Self-heal Daemon on localhost               N/A       N/A        Y       130186
Self-heal Daemon on srvgluspvlsu71          N/A       N/A        Y       20009
Self-heal Daemon on srvgluspvlsu72.example.
com                                         N/A       N/A        Y       12300

Task Status of Volume volQXSCCINTP_data
------------------------------------------------------------------------------
There are no active volume tasks
""".strip()


def test_invalid():
    with pytest.raises(ParseException) as e:
        GlusterVolStatus(context_wrap(VOL_STATUS_BAD))
    assert "Unable to parse gluster volume status: []" in str(e)


def test_valid():
    parser_result = GlusterVolStatus(context_wrap(VOL_STATUS_GOOD))
    parser_data = parser_result.data
    assert list(parser_data.keys()) == ["test_vol"]
    assert parser_data["test_vol"][0] == {
        'Gluster_process': 'Brick 172.17.18.42:/home/brick',
        'RDMA_Port': '0',
        'TCP_Port': '49152',
        'Pid': '26685',
        'Online': 'Y'
    }
    assert parser_data["test_vol"][1]["Online"] == 'Y'
    assert parser_data["test_vol"][1]["TCP_Port"] == '49152'
    assert parser_data["test_vol"][1]["Pid"] == '27094'
    assert parser_data["test_vol"][2] == {
            'Gluster_process': 'Brick 172.17.18.44:/home/brick',
            'RDMA_Port': '0',
            'TCP_Port': '49152',
            'Pid': '27060',
            'Online': 'Y'
    }
    assert parser_data["test_vol"][3] == {
            'Online': 'Y',
            'TCP_Port': 'N/A',
            'RDMA_Port': 'N/A',
            'Pid': '7805',
            'Gluster_process': 'Self-heal Daemon on localhost'
    }
    assert parser_data["test_vol"][4] == {
            'Online': 'Y',
            'TCP_Port': 'N/A',
            'RDMA_Port': 'N/A',
            'Pid': '33400',
            'Gluster_process': 'Self-heal Daemon on 172.17.18.44'
    }
    Self_heal_Daemon3_data = parser_data["test_vol"][5]
    assert Self_heal_Daemon3_data['RDMA_Port'] == 'N/A'


def test_multiple_valid():
    parser_result = GlusterVolStatus(context_wrap(VOL_STATUS_MULTIPLE_GOOD))
    parser_data = parser_result.data
    assert list(parser_data.keys()) == ["test_vol", "test_vol_2"]
    assert parser_data["test_vol_2"][0] == {
            'Gluster_process': 'Brick 172.17.18.42:/home/brick',
            'RDMA_Port': '0',
            'TCP_Port': '49152',
            'Pid': '6685',
            'Online': 'Y'
    }
    assert parser_data["test_vol_2"][1]["Online"] == 'Y'
    assert parser_data["test_vol_2"][1]["TCP_Port"] == '49152'
    assert parser_data["test_vol_2"][1]["Pid"] == '33094'
    assert parser_data["test_vol_2"][2] == {
            'Gluster_process': 'Brick 172.17.18.44:/home/brick',
            'RDMA_Port': '0',
            'TCP_Port': '49152',
            'Pid': '2060',
            'Online': 'Y'
    }
    assert parser_data["test_vol_2"][3] == {
            'Gluster_process': 'Self-heal Daemon on localhost',
            'RDMA_Port': 'N/A',
            'Pid': '7805',
            'TCP_Port': 'N/A',
            'Online': 'Y'}


def test_long_hostnames():
    result = GlusterVolStatus(context_wrap(MULTIPLE_VOL_STATUS_LONG_HOSTNAME))
    assert len(result.data) == 2
    assert len(result.data['volQXCHISTPI_log']) == 6
    assert len(result.data['volQXSCCINTP_data']) == 6
    assert result.data['volQXSCCINTP_data'][0]['Gluster_process'] == 'Brick srvgluspvlsu71.example.com:/gluster/QXSCCINTP/data/brick'

# --------------------------------------END------------------------------------


def test_doc():
    INFO = TRACKING_VALID
    STATUS = VOL_STATUS_GOOD
    env = {
            "parser_result_v_info": GlusterVolInfo(context_wrap(INFO)),
            "parser_result_v_status": GlusterVolStatus(context_wrap(STATUS))
    }
    failed, total = doctest.testmod(gluster_vol, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.parsers import nfnetlink_queue, ParseException
from insights.tests.parsers import skip_exception_check
from insights.tests import context_wrap


NFNETLINK_QUEUE = """
    0  -4423     0 2 65535     0     0       22  1
    1  -4424     0 2 65535     0     0       27  1
    2  -4425     0 2 65535     0     0       17  1
    3  -4426     0 2 65535     0     0       14  1
    4  -4427     0 2 65535     0     0       22  1
    5  -4428     0 2 65535     0     0       16  1
""".strip()

CORRUPT_NFNETLINK_QUEUE_1 = """
    0  -4423     0 2 65535     0     0       22  1
    1  -4424     0 2 6553
    2  -4425     0 2 65535     0     0       17  1
    3  -4426     0 2 65535     0     0       14  1
    4  -4427     0 2 65535     0     0       22  1
    5  -4428     0 2 65535     0     0       16  1
""".strip()

CORRUPT_NFNETLINK_QUEUE_2 = """
    0  -4423     0 2 65535     0     0       22  1
    1  -4424     0 2 astring   0     0       27  1
    2  -4425     0 2 65535     0     0       17  1
    3  -4426     0 2 65535     0     0       14  1
    4  -4427     0 2 65535     0     0       22  1
    5  -4428     0 2 65535     0     0       16  1
""".strip()

NFNETLINK_DOC = """
0  -4423     0 2 65535     0     0       22  1
1  -4424     0 2 65535     0     0       27  1
""".strip()


def test_nfnetlink_doc_examples():
    env = {
        'nf': nfnetlink_queue.NfnetLinkQueue(context_wrap(NFNETLINK_DOC))
    }
    failed, total = doctest.testmod(nfnetlink_queue, globs=env)
    assert failed == 0


def test_parse_content():
    nfnet_link_queue = nfnetlink_queue.NfnetLinkQueue(context_wrap(NFNETLINK_QUEUE))
    row = nfnet_link_queue.data[0]
    assert row["queue_number"] == 0
    assert row["peer_portid"] == -4423
    assert row["queue_total"] == 0
    assert row["copy_mode"] == 2
    assert row["copy_range"] == 65535
    assert row["queue_dropped"] == 0
    assert row["user_dropped"] == 0
    assert row["id_sequence"] == 22

    row = nfnet_link_queue.data[5]
    assert row["queue_number"] == 5
    assert row["peer_portid"] == -4428
    assert row["queue_total"] == 0
    assert row["copy_mode"] == 2
    assert row["copy_range"] == 65535
    assert row["queue_dropped"] == 0
    assert row["user_dropped"] == 0
    assert row["id_sequence"] == 16


def test_missing_columns():
    with pytest.raises(ParseException):
        nfnetlink_queue.NfnetLinkQueue(context_wrap(CORRUPT_NFNETLINK_QUEUE_1))


def test_wrong_type():
    with pytest.raises(ValueError):
        nfnetlink_queue.NfnetLinkQueue(context_wrap(CORRUPT_NFNETLINK_QUEUE_2))


def test_empty_content():
    skip_exception_check(nfnetlink_queue.NfnetLinkQueue)



import doctest
import pytest

from insights.parsers import lsvmbus
from insights.parsers import SkipException
from insights.tests import context_wrap


OUTPUT = """
VMBUS ID 18: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
        Device_ID = {47505500-0001-0000-3130-444531303244}
        Sysfs path: /sys/bus/vmbus/devices/47505500-0001-0000-3130-444531303244
        Rel_ID=18, target_cpu=0
VMBUS ID 26: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
        Device_ID = {47505500-0002-0000-3130-444531303244}
        Sysfs path: /sys/bus/vmbus/devices/47505500-0002-0000-3130-444531303244
        Rel_ID=26, target_cpu=0
VMBUS ID 73: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
        Device_ID = {47505500-0003-0001-3130-444531303244}
        Sysfs path: /sys/bus/vmbus/devices/47505500-0003-0001-3130-444531303244
        Rel_ID=73, target_cpu=0
VMBUS ID 74: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
        Device_ID = {47505500-0004-0001-3130-444531303244}
        Sysfs path: /sys/bus/vmbus/devices/47505500-0004-0001-3130-444531303244
        Rel_ID=74, target_cpu=0
""".strip()


def test_lsvmbus():
    output = lsvmbus.LsvmBus(context_wrap(OUTPUT))
    assert len(output.devices) == 4
    assert output.devices[0].get('vmbus_id', None) == '18'
    assert output.devices[0].get('device_id', None) == '47505500-0001-0000-3130-444531303244'
    assert output.devices[0].get('rel_id', None) == '18'
    assert output.devices[0].get('sysfs_path', None) == '/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531303244'
    assert output.devices[0].get('target_cpu', None) == '0'

    with pytest.raises(SkipException):
        assert lsvmbus.LsvmBus(context_wrap("")) is None


def test_docs():
    env = {'lsvmbus': lsvmbus.LsvmBus(context_wrap(OUTPUT))}
    failed, total = doctest.testmod(lsvmbus, globs=env)
    assert failed == 0



from insights.tests import context_wrap
from insights.parsers.sysconfig import OracleasmSysconfig

ORACLEASM_SYSCONFIG = """
#
# This is a configuration file for automatic loading of the Oracle
# Automatic Storage Management library kernel driver.  It is generated
# By running /etc/init.d/oracleasm configure.  Please use that method
# to modify this file
#

# ORACLEASM_ENABELED: 'true' means to load the driver on boot.
ORACLEASM_ENABLED=true

# ORACLEASM_UID: Default user owning the /dev/oracleasm mount point.
ORACLEASM_UID=oracle

# ORACLEASM_GID: Default group owning the /dev/oracleasm mount point.
ORACLEASM_GID=oinstall

# ORACLEASM_SCANBOOT: 'true' means scan for ASM disks on boot.
ORACLEASM_SCANBOOT=true

# ORACLEASM_SCANORDER: Matching patterns to order disk scanning
ORACLEASM_SCANORDER="dm"

# ORACLEASM_SCANEXCLUDE: Matching patterns to exclude disks from scan
ORACLEASM_SCANEXCLUDE="sd"
""".strip()


def test_sysconfig_oracleasm():
    result = OracleasmSysconfig(context_wrap(ORACLEASM_SYSCONFIG))
    assert result["ORACLEASM_SCANORDER"] == 'dm'
    assert result.get("ORACLEASM_SCANBOOT") == 'true'
    assert result.get("NONEXISTENT_VAR") is None
    assert "NONEXISTENT_VAR" not in result
    assert "ORACLEASM_SCANEXCLUDE" in result



"""
NeutronMl2Conf - file ``/etc/neutron/plugins/ml2/ml2_conf.ini``
===============================================================
"""
from insights.core import IniConfigFile
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

add_filter(Specs.neutron_ml2_conf, ["["])


@parser(Specs.neutron_ml2_conf)
class NeutronMl2Conf(IniConfigFile):
    """
    Neutron ML2 configuration parser class, based on the ``IniConfigFile`` class.

    Sample input data is in the format::

        [DEFAULT]
        debug = false
        verbose = true
        use_syslog = false

        [ml2]
        type_drivers = local,flat,vlan,gre,vxlan,geneve
        mechanism_drivers = openvswitch

        [securitygroup]
        firewall_driver = iptables_hybrid
        enable_security_group = true

    Examples:
        >>> type(neutron_ml2_conf)
        <class 'insights.parsers.neutron_ml2_conf.NeutronMl2Conf'>
        >>> neutron_ml2_conf.has_option("ml2", "type_drivers")
        True
        >>> neutron_ml2_conf.get("ml2", "mechanism_drivers") == "openvswitch"
        True
    """
    pass



from __future__ import print_function
from insights.parsers.uname import Uname
from insights.core.plugins import make_fail
from insights.tests import context_wrap, InputData, run_test
from insights.specs import Specs

from insights.plugins import vulnerable_kernel

ERROR_KEY = 'VULNERABLE_KERNEL'

UNAME_TEMPLATE = "Linux testhost1 %s #1 SMP Tue Jan 29 11:47:41 EST 2013 x86_64 x86_64 x86_64 GNU/Linux"

NOT_VULNERABLE = [
    '2.4.32-100.el6.x86_64',
    '2.6.32-430.el6.x86_64',
    '2.6.32-431.11.2.el6.x86_64',
    '2.6.32-431.11.3.el6.x86_64',
    '2.6.32-432.el6.x86_64',
    '2.7.12-200.el6.x86_64',
]


VULNERABLE = [
    '2.6.32-431.el6.x86_64',
    '2.6.32-431.10.1.el6.x86_64',
    '2.6.32-431.11.1.el6.x86_64',
]


def test_vulnerable_kernel():
    for kernel in NOT_VULNERABLE:
        uname_line = UNAME_TEMPLATE % kernel
        result = vulnerable_kernel.report(Uname(context_wrap(uname_line)))
        expected = None
        if not (result == expected):
            print(result)
            print(expected)
            assert result == expected
            assert False
    for kernel in VULNERABLE:
        uname_line = UNAME_TEMPLATE % kernel
        result = vulnerable_kernel.report(Uname(context_wrap(uname_line)))
        expected = make_fail(ERROR_KEY, kernel=kernel)
        if not (result == expected):
            print(result)
            print(expected)
            assert result == expected
            assert False


def generate_inputs(things):
    for kernel in things:
        uname_line = UNAME_TEMPLATE % kernel
        i = InputData()
        i.add(Specs.uname, uname_line)
        yield (kernel, i)


def test_vulnerable_kernel_integration():
    comp = vulnerable_kernel.report
    for kernel, i in generate_inputs(VULNERABLE):
        expected = make_fail(ERROR_KEY, kernel=kernel)
        run_test(comp, i, expected)

    for _, i in generate_inputs(NOT_VULNERABLE):
        run_test(comp, i, None)



"""
Custom datasources for cloud initialization information
"""
import json
import yaml

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.filters import get_filters
from insights.core.spec_factory import DatasourceProvider, simple_file
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by cloud_init datasources """

    cloud_cfg_input = simple_file("/etc/cloud/cloud.cfg")
    """ Returns the contents of the file ``/etc/cloud/cloud.cfg`` """


@datasource(LocalSpecs.cloud_cfg_input, HostContext)
def cloud_cfg(broker):
    """
    This datasource provides configuration collected from ``/etc/cloud/cloud.cfg``.

    Typical content of ``/etc/cloud/cloud.cfg`` file is::

        #cloud-config
        users:
          - name: demo
            ssh-authorized-keys:
              - key_one
              - key_two
            passwd: $6$j212wezy$7H/1LT4f9/N3wpgNunhsIqtMj62OKiS3nyNwuizouQc3u7MbYCarYeAHWYPYb2FT.lbioDm2RrkJPb9BZMN1O/

        ssh_deletekeys: 1

        network:
            version: 1
            config:
            - type: physical
                name: eth0
                subnets:
                  - type: dhcp
                  - type: dhcp6

        system_info:
            default_user:
            name: user2
            plain_text_passwd: 'someP@assword'
            home: /home/user2

        debug:
            output: /var/log/cloud-init-debug.log
            verbose: true

    Note:
        This datasource may be executed using the following command:

        ``insights cat --no-header cloud_cfg``

    Sample output in JSON format::

        {
          "ssh_deletekeys": 1,
          "network": {
            "version": 1,
            "config": [
              {
                "type": "physical",
                "name": "eth0",
                "subnets": [
                  {
                    "type": "dhcp"
                  },
                  {
                    "type": "dhcp6"
                  }
                ]
              }
            ]
          },
          "debug": {
            "output": "/var/log/cloud-init-debug.log",
            "verbose": true
          }
        }

    Returns:
        str: JSON string after removing the sensitive information.

    Raises:
        SkipComponent: When the path does not exist or any exception occurs.
    """
    relative_path = '/etc/cloud/cloud.cfg'
    try:
        filters = get_filters(Specs.cloud_cfg)
        content = broker[LocalSpecs.cloud_cfg_input].content
        if content and filters:
            result = dict()
            content = yaml.load('\n'.join(content), Loader=yaml.SafeLoader)
            if isinstance(content, dict):
                # remove sensitive data
                content.pop('users', None)
                content.pop('system_info', None)
                # apply filters
                for item in filters:
                    if item in content:
                        result[item] = content[item]

                if result:
                    result = dict(sorted(result.items(), key=lambda x: x[0]))
                    return DatasourceProvider(content=json.dumps(result), relative_path=relative_path)
            raise SkipComponent("Invalid YAML format")
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent



"""
OpenStack gnocchi related files or commands
===========================================

Module for the processing of gnocchi related files and commands.
This module provides parsers:

GnocchiConf - file ``/etc/gnocchi.gnocchi.conf``
------------------------------------------------

GnocchiMetricdLog - file ``gnocchi-metricd.log`` or ``metricd.log``
-------------------------------------------------------------------
"""

from insights.core import IniConfigFile, LogFileOutput
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs


add_filter(Specs.gnocchi_conf, "[")


@parser(Specs.gnocchi_conf)
class GnocchiConf(IniConfigFile):
    """
    Gnocchi configuration parser class, based on the ``IniConfigFile`` class.

    The gnocchi configuration file is a standard '.ini' file and this parser uses
    the ``IniConfigFile`` class to read it.

    Sample configuration::

        [DEFAULT]
        log_dir = /var/log/gnocchi
        [api]
        auth_mode = keystone
        max_limit = 1000
        [archive_policy]
        [indexer]
        url = mysql+pymysql://gnocchi:exampleabckeystring@192.168.0.1/gnocchi?charset=utf8
        [metricd]
        workers = 2
        [oslo_middleware]
        [oslo_policy]
        policy_file = /etc/gnocchi/policy.json
        [statsd]
        resource_id = 5e3fcbe2-7aab-475d-b42c-abcdefgh
        user_id = e0ca4711-1128-422c-abd6-abcdefgh
        project_id = af0c88e8-90d8-4795-9efe-abcdefgh
        archive_policy_name = high
        flush_delay = 10
        [storage]
        driver = file
        file_basepath = /var/lib/gnocchi
        [keystone_authtoken]
        auth_uri=http://192.168.0.1:5000/v3
        auth_type=password
        auth_version=v3
        auth_url=http://192.168.0.1:35357
        username=gnocchi
        password=yourpassword23432
        user_domain_name=Default
        project_name=services
        project_domain_name=Default


    Examples:
        >>> type(conf)
        <class 'insights.parsers.gnocchi.GnocchiConf'>
        >>> sorted(conf.sections()) == sorted([u'api', u'archive_policy', u'indexer', u'metricd', u'oslo_middleware', u'oslo_policy', u'statsd', u'storage', u'keystone_authtoken'])
        True
        >>> 'storage' in conf
        True
        >>> conf.has_option('indexer', 'url')
        True
        >>> conf.get('indexer', 'url') == u'mysql+pymysql://gnocchi:exampleabckeystring@192.168.0.1/gnocchi?charset=utf8'
        True
        >>> conf.getint("statsd", "flush_delay")
        10
    """
    pass


@parser(Specs.gnocchi_metricd_log)
class GnocchiMetricdLog(LogFileOutput):
    """
    Provide access to metricd.log using the LogFileOutput parser class.


    This log file is a standard log parser based on the LogFileOutput class.
    From OpenStack 12, the log ``metricd.log`` is renamed to ``gnocchi-metricd.log``

    Sample log file::

        2017-04-12 03:10:53.076 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
        2017-04-12 03:12:53.078 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
        2017-04-12 03:14:53.080 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
        2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock

    Examples:

        >>> 'measurements bundles across 0 metrics wait to be processed' in log
        True
        >>> log.get('ERROR')
        [{'raw_message': '2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock'}]
        >>> from datetime import datetime
        >>> list(log.get_after(datetime(2017, 4, 12, 19, 36, 38)))
        [{'raw_message': '2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock'}]

    """
    pass



from insights.parsers.smartctl import SMARTctl
from insights.parsers import ParseException
from insights.tests import context_wrap

import pytest

STANDARD_DRIVE = """
smartctl 6.2 2013-07-26 r3841 [x86_64-linux-3.10.0-267.el7.x86_64] (local build)
Copyright (C) 2002-13, Bruce Allen, Christian Franke, www.smartmontools.org

=== START OF INFORMATION SECTION ===
Device Model:     ST500LM021-1KJ152
Serial Number:    W620AT02
LU WWN Device Id: 5 000c50 07817bb36
Firmware Version: 0002LIM1
User Capacity:    500,107,862,016 bytes [500 GB]
Sector Sizes:     512 bytes logical, 4096 bytes physical
Rotation Rate:    7200 rpm
Device is:        Not in smartctl database [for details use: -P showall]
ATA Version is:   ATA8-ACS T13/1699-D revision 4
SATA Version is:  SATA 3.0, 6.0 Gb/s (current: 6.0 Gb/s)
Local Time is:    Fri Sep 16 14:10:10 2016 AEST
SMART support is: Available - device has SMART capability.
SMART support is: Enabled

=== START OF READ SMART DATA SECTION ===
SMART overall-health self-assessment test result: PASSED

General SMART Values:
Offline data collection status:  (0x00) Offline data collection activity
                    was never started.
                    Auto Offline Data Collection: Disabled.
Self-test execution status:      (   0) The previous self-test routine completed
                    without error or no self-test has ever
                    been run.
Total time to complete Offline
data collection:        (    0) seconds.
Offline data collection
capabilities:            (0x73) SMART execute Offline immediate.
                    Auto Offline data collection on/off support.
                    Suspend Offline collection upon new
                    command.
                    No Offline surface scan supported.
                    Self-test supported.
                    Conveyance Self-test supported.
                    Selective Self-test supported.
SMART capabilities:            (0x0003) Saves SMART data before entering
                    power-saving mode.
                    Supports SMART auto save timer.
Error logging capability:        (0x01) Error logging supported.
                    General Purpose Logging supported.
Short self-test routine
recommended polling time:    (   1) minutes.
Extended self-test routine
recommended polling time:    (  78) minutes.
Conveyance self-test routine
recommended polling time:    (   2) minutes.
SCT capabilities:          (0x1031) SCT Status supported.
                    SCT Feature Control supported.
                    SCT Data Table supported.

SMART Attributes Data Structure revision number: 10
Vendor Specific SMART Attributes with Thresholds:
ID# ATTRIBUTE_NAME          FLAG     VALUE WORST THRESH TYPE      UPDATED  WHEN_FAILED RAW_VALUE
  1 Raw_Read_Error_Rate     0x000f   118   099   034    Pre-fail  Always       -       179599704
  3 Spin_Up_Time            0x0003   098   098   000    Pre-fail  Always       -       0
  4 Start_Stop_Count        0x0032   100   100   020    Old_age   Always       -       546
  5 Reallocated_Sector_Ct   0x0033   100   100   036    Pre-fail  Always       -       0
  7 Seek_Error_Rate         0x000f   078   060   030    Pre-fail  Always       -       61310462
  9 Power_On_Hours          0x0032   096   096   000    Old_age   Always       -       4273 (5 89 0)
 10 Spin_Retry_Count        0x0013   100   100   097    Pre-fail  Always       -       0
 12 Power_Cycle_Count       0x0032   100   100   020    Old_age   Always       -       542
184 End-to-End_Error        0x0032   100   100   099    Old_age   Always       -       0
187 Reported_Uncorrect      0x0032   100   100   000    Old_age   Always       -       0
188 Command_Timeout         0x0032   100   099   000    Old_age   Always       -       1
189 High_Fly_Writes         0x003a   100   100   000    Old_age   Always       -       0
190 Airflow_Temperature_Cel 0x0022   059   050   045    Old_age   Always       -       41 (Min/Max 21/41)
191 G-Sense_Error_Rate      0x0032   100   100   000    Old_age   Always       -       19
192 Power-Off_Retract_Count 0x0032   100   100   000    Old_age   Always       -       8
193 Load_Cycle_Count        0x0032   099   099   000    Old_age   Always       -       3009
194 Temperature_Celsius     0x0022   041   050   000    Old_age   Always       -       41 (0 11 0 0 0)
196 Reallocated_Event_Count 0x000f   096   096   030    Pre-fail  Always       -       4258 (30941 0)
197 Current_Pending_Sector  0x0012   100   100   000    Old_age   Always       -       0
198 Offline_Uncorrectable   0x0010   100   100   000    Old_age   Offline      -       0
199 UDMA_CRC_Error_Count    0x003e   200   200   000    Old_age   Always       -       0
254 Free_Fall_Sensor        0x0032   100   100   000    Old_age   Always       -       0

SMART Error Log Version: 1
No Errors Logged

SMART Self-test log structure revision number 1
Num  Test_Description    Status                  Remaining  LifeTime(hours)  LBA_of_first_error
# 1  Short offline       Completed without error       00%      2315         -

SMART Selective self-test log data structure revision number 1
 SPAN  MIN_LBA  MAX_LBA  CURRENT_TEST_STATUS
    1        0        0  Not_testing
    2        0        0  Not_testing
    3        0        0  Not_testing
    4        0        0  Not_testing
    5        0        0  Not_testing
Selective self-test flags (0x0):
  After scanning selected spans, do NOT read-scan remainder of disk.
If Selective self-test is pending on power-up, resume after 0 minute delay.

"""


def test_standard_drive():
    data = SMARTctl(context_wrap(
        STANDARD_DRIVE, path='sos_commands/ata/smartctl_-a_.dev.sda'
    ))

    # Device assertions
    assert data.device == '/dev/sda'

    # Information assertions
    assert data.information['Device Model'] == 'ST500LM021-1KJ152'
    assert data.information['Serial Number'] == 'W620AT02'
    assert data.information['LU WWN Device Id'] == '5 000c50 07817bb36'
    assert data.information['Firmware Version'] == '0002LIM1'
    assert data.information['User Capacity'] == '500,107,862,016 bytes [500 GB]'
    assert data.information['Sector Sizes'] == '512 bytes logical, 4096 bytes physical'
    assert data.information['Rotation Rate'] == '7200 rpm'
    assert data.information['Device is'] == 'Not in smartctl database [for details use: -P showall]'
    assert data.information['ATA Version is'] == 'ATA8-ACS T13/1699-D revision 4'
    assert data.information['SATA Version is'] == 'SATA 3.0, 6.0 Gb/s (current: 6.0 Gb/s)'
    assert data.information['Local Time is'] == 'Fri Sep 16 14:10:10 2016 AEST'
    # assert data.information['SMART support is'] == 'Available - device has SMART capability.'
    # Note: this key turns up twice - therefore new keys overwrite old
    assert data.information['SMART support is'] == 'Enabled'

    # Health assertions
    assert data.health == 'PASSED'

    # SMART Value assertions
    assert data.values['Offline data collection status'] == '0x00'
    assert data.values['Self-test execution status'] == '0'
    assert data.values['Total time to complete Offline data collection'] == '0'
    assert data.values['Offline data collection capabilities'] == '0x73'
    assert data.values['SMART capabilities'] == '0x0003'
    assert data.values['Error logging capability'] == '0x01'
    assert data.values['Short self-test routine recommended polling time'] == '1'
    assert data.values['Extended self-test routine recommended polling time'] == '78'
    assert data.values['Conveyance self-test routine recommended polling time'] == '2'
    assert data.values['SCT capabilities'] == '0x1031'
    assert data.values['SMART Attributes Data Structure revision number'] == '10'

    # Attribute assertions
    # Don't bother to test every single key for every single attribute - test
    # one, and then test the variations
    assert data.attributes['Raw_Read_Error_Rate']['id'] == '1'
    assert data.attributes['Raw_Read_Error_Rate']['flag'] == '0x000f'
    assert data.attributes['Raw_Read_Error_Rate']['value'] == '118'
    assert data.attributes['Raw_Read_Error_Rate']['worst'] == '099'
    assert data.attributes['Raw_Read_Error_Rate']['threshold'] == '034'
    assert data.attributes['Raw_Read_Error_Rate']['type'] == 'Pre-fail'
    assert data.attributes['Raw_Read_Error_Rate']['updated'] == 'Always'
    assert data.attributes['Raw_Read_Error_Rate']['when_failed'] == '-'
    assert data.attributes['Raw_Read_Error_Rate']['raw_value'] == '179599704'

    assert data.attributes['Start_Stop_Count']['type'] == 'Old_age'
    assert data.attributes['Power_On_Hours']['raw_value'] == '4273 (5 89 0)'
    assert data.attributes['Airflow_Temperature_Cel']['raw_value'] == '41 (Min/Max 21/41)'
    assert data.attributes['Offline_Uncorrectable']['updated'] == 'Offline'

    # No parsing leftovers
    assert not hasattr(data, 'full_line')
    # No legacy data
    assert not hasattr(data, 'info')


def test_bad_device():
    # Bad device causes a parse error
    with pytest.raises(ParseException) as exc:
        assert SMARTctl(context_wrap(
            STANDARD_DRIVE, path='sos_commands/ata/smartctl_-a'
        )) is None
    assert 'Cannot parse device name from path ' in str(exc)


CISCO_DRIVE = """
smartctl 5.43 2012-06-30 r3573 [x86_64-linux-2.6.32-573.8.1.el6.x86_64] (local build)
Copyright (C) 2002-12 by Bruce Allen, http://smartmontools.sourceforge.net

Vendor:               Cisco
Product:              UCSC-MRAID12G
Revision:             4.27
User Capacity:        898,999,779,328 bytes [898 GB]
Logical block size:   512 bytes
Logical Unit id:      0x678da6e715b756401d552c0c04e4953b
Serial number:        003b95e4040c2c551d4056b715e7a68d
Device type:          disk
Local Time is:        Wed Dec 16 21:29:59 2015 EST
Device does not support SMART

Error Counter logging not supported
Device does not support Self Test logging
"""  # noqa


def test_cisco_drive():
    data = SMARTctl(context_wrap(
        CISCO_DRIVE, path='sos_commands/ata/smartctl_-a_.dev.sdb'
    ))

    # Device assertions
    assert data.device == '/dev/sdb'

    # Information assertions
    assert data.information['Vendor'] == 'Cisco'
    assert data.information['Product'] == 'UCSC-MRAID12G'
    assert data.information['Revision'] == '4.27'
    assert data.information['User Capacity'] == '898,999,779,328 bytes [898 GB]'
    assert data.information['Logical block size'] == '512 bytes'
    assert data.information['Logical Unit id'] == '0x678da6e715b756401d552c0c04e4953b'
    assert data.information['Serial number'] == '003b95e4040c2c551d4056b715e7a68d'
    assert data.information['Device type'] == 'disk'
    assert data.information['Local Time is'] == 'Wed Dec 16 21:29:59 2015 EST'

    # Unstructured information assertions
    assert data.information['SMART support is'] == 'Not supported'
    assert data.information['Error Counter logging'] == 'Not supported'
    assert data.information['Self Test logging'] == 'Not supported'

    # Everything else is blank
    assert data.health == 'not parsed'
    assert data.values == {}
    assert data.attributes == {}


NETAPP_DRIVE = """
smartctl 5.43 2012-06-30 r3573 [x86_64-linux-2.6.32-573.8.1.el6.x86_64] (local build)
Copyright (C) 2002-12 by Bruce Allen, http://smartmontools.sourceforge.net

Vendor:               NETAPP
Product:              LUN
Revision:             820a
User Capacity:        5,243,081,326,592 bytes [5.24 TB]
Logical block size:   512 bytes
Logical Unit id:      0x60a9800044312d364d5d4478753370620x5d447875337062000a980044312d364d
Serial number:        D1-6M]Dxu3pb
Device type:          disk
Transport protocol:   iSCSI
Local Time is:        Wed Dec 16 21:29:59 2015 EST
Device supports SMART and is Enabled
Temperature Warning Disabled or Not Supported
SMART Health Status: OK

Error Counter logging not supported
Device does not support Self Test logging

"""  # noqa


def test_netapp_drive():
    data = SMARTctl(context_wrap(
        NETAPP_DRIVE, path='sos_commands/ata/smartctl_-a_.dev.sdc'
    ))

    # Device assertions
    assert data.device == '/dev/sdc'

    # Information assertions
    assert data.information['Vendor'] == 'NETAPP'
    assert data.information['Product'] == 'LUN'
    assert data.information['Revision'] == '820a'
    assert data.information['User Capacity'] == '5,243,081,326,592 bytes [5.24 TB]'
    assert data.information['Logical block size'] == '512 bytes'
    assert data.information['Logical Unit id'] == '0x60a9800044312d364d5d4478753370620x5d447875337062000a980044312d364d'
    assert data.information['Serial number'] == 'D1-6M]Dxu3pb'
    assert data.information['Device type'] == 'disk'
    assert data.information['Transport protocol'] == 'iSCSI'
    assert data.information['Local Time is'] == 'Wed Dec 16 21:29:59 2015 EST'
    assert data.information['SMART Health Status'] == 'OK'

    # Unstructured information assertions
    assert data.information['SMART support is'] == 'Enabled'
    assert data.information['Temperature Warning'] == 'Disabled or Not Supported'
    assert data.information['Error Counter logging'] == 'Not supported'
    assert data.information['Self Test logging'] == 'Not supported'

    # Everything else is blank
    assert data.health == 'not parsed'
    assert data.values == {}
    assert data.attributes == {}



from insights.parsers import yum
from insights.parsers.yum import YumRepoList
from insights.parsers import SkipException, ParseException
from insights.tests import context_wrap
import doctest
import pytest

YUM_REPOLIST_CONTENT = """
Loaded plugins: product-id, search-disabled-repos, subscription-manager
repo id                                         repo name                status
rhel-7-server-rpms/7Server/x86_64               Red Hat Enterprise Linux 10415
rhel-7-server-satellite-6.1-rpms/x86_64         Red Hat Satellite 6.1 (f   660
rhel-7-server-satellite-capsule-6.1-rpms/x86_64 Red Hat Satellite Capsul   265
rhel-server-rhscl-7-rpms/7Server/x86_64         Red Hat Software Collect  4571
repolist: 15911

""".strip()

YUM_REPOLIST_CONTENT_NOPLUGINS = """
repo id                                         repo name                status
rhel-7-server-rpms/7Server/x86_64               Red Hat Enterprise Linux 10415
rhel-7-server-satellite-6.1-rpms/x86_64         Red Hat Satellite 6.1 (f   660
rhel-7-server-satellite-capsule-6.1-rpms/x86_64 Red Hat Satellite Capsul   265
rhel-server-rhscl-7-rpms/7Server/x86_64         Red Hat Software Collect  4571
repolist: 15911

""".strip()

YUM_REPOLIST_CONTENT_EUS = """
Loaded plugins: product-id, rhnplugin, security, subscription-manager
Updating certificate-based repositories.
repo id                              repo name                            status
clone-6u5-server-x86_64              clone-6u5-server-x86_64              3,787
rhel-x86_64-server-6.2.aus           Red Hot Enterprise Linu              5,509
repolist: 3,787

""".strip()

YUM_REPOLIST_CONTENT_MISSING_STATUS = """
Loaded plugins: product-id, rhnplugin, security, subscription-manager
Updating certificate-based repositories.
repo id                              repo name                            status
clone-6u5-server-x86_64              clone-6u5-server-x86_64
repolist: 3,787
"""

YUM_REPOLIST_CONTENT_OUT_OF_DATE = """
Loaded plugins: product-id, search-disabled-repos, subscription-manager
Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
repo id                              repo name                                          status
!rhel-7-server-extras-rpms/x86_64    Red Hat Enterprise Linux 7 Server - Extras (RPMs)  877
!rhel-7-server-rpms/7Server/x86_64   Red Hat Enterprise Linux 7 Server (RPMs)           20,704
repolist: 21,581
"""

YUM_REPOLIST_CONTENT_OUT_OF_DATE_AND_NOT_MATCH_METADATA = """
Loaded plugins: product-id, search-disabled-repos, subscription-manager
Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
repo id                              repo name                                          status
!rhel-7-server-extras-rpms/x86_64    Red Hat Enterprise Linux 7 Server - Extras (RPMs)  877
*rhel-7-server-rpms/7Server/x86_64   Red Hat Enterprise Linux 7 Server (RPMs)           20,704
repolist: 21,581
"""

YUM_REPO_NO_REPO_NAME = """
Loaded plugins: enabled_repos_upload, package_upload, priorities, product-id,
              : search-disabled-repos, security, subscription-manager
repo id                                                                               status
LME_EPEL_6_x86_64                                                                     26123
LME_HP_-_Software_Delivery_Repository_Firmware_Pack_for_ProLiant_-_6Server_-_Current   1163
LME_HP_-_Software_Delivery_Repository_Scripting_Took_Kit_-_6Server                       17
LME_HP_-_Software_Delivery_Repository_Service_Pack_for_ProLiant_-_6Server_-_Current    1861
LME_HP_-_Software_Delivery_Repository_Smart_Update_Manager_-_6Server                     21
rhel-6-server-optional-rpms                                                           11358
rhel-6-server-rpms                                                                    19753
repolist: 60296
Uploading Enabled Reposistories Report
Plugin "search-disabled-repos" requires API 2.7. Supported API is 2.6.
Loaded plugins: priorities, product-id
""".strip()

YUM_REPOLIST_CONTENT_MISSING_END = """
Loaded plugins: product-id, rhnplugin, security, subscription-manager
Updating certificate-based repositories.
repo id                              repo name                            status
clone-6u5-server-x86_64              clone-6u5-server-x86_64              1234
"""

YUM_REPOLIST_CONTENT_EMPTY = """
repo id                              repo name                            status
"""

YUM_REPOLIST_DOC = """
Loaded plugins: langpacks, product-id, search-disabled-repos, subscription-manager
repo id                                             repo name                                                                                                    status
rhel-7-server-e4s-rpms/x86_64                       Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)                                 12,250
!rhel-ha-for-rhel-7-server-e4s-rpms/x86_64          Red Hat Enterprise Linux High Availability (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)         272
*rhel-sap-hana-for-rhel-7-server-e4s-rpms/x86_64    RHEL for SAP HANA (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)                                   21
repolist: 12,768
""".strip()

YUM_REPOLIST_RHEL8 = """
Updating Subscription Management repositories.
Repository packages-microsoft-com-prod is listed more than once in the configuration
repo id                                                                           repo name
fast-datapath-for-rhel-8-x86_64-rpms                                              Fast Datapath for RHEL 8 x86_64 (RPMs)
packages-microsoft-com-mssql-server-2019                                          packages-microsoft-com-mssql-server-2019
packages-microsoft-com-prod                                                       packages-microsoft-com-prod
rhel-8-for-x86_64-appstream-rpms                                                  Red Hat Enterprise Linux 8 for x86_64 - AppStream (RPMs)
rhel-8-for-x86_64-baseos-rpms                                                     Red Hat Enterprise Linux 8 for x86_64 - BaseOS (RPMs)
rhel-8-for-x86_64-highavailability-rpms                                           Red Hat Enterprise Linux 8 for x86_64 - High Availability (RPMs)
rhel-8-for-x86_64-sap-solutions-rpms                                              Red Hat Enterprise Linux 8 for x86_64 - SAP Solutions (RPMs)
rhel-8-for-x86_64-supplementary-rpms                                              Red Hat Enterprise Linux 8 for x86_64 - Supplementary (RPMs)
satellite-tools-6.7-for-rhel-8-x86_64-rpms                                        Red Hat Satellite Tools 6.7 for RHEL 8 x86_64 (RPMs)
""".strip()

YUM_REPOLIST_DOC_NO_REPONAME = """
Loaded plugins: package_upload, product-id, search-disabled-repos, security, subscription-manager
repo id                                                                               status
LME_EPEL_6_x86_64                                                                        26123
LME_FSMLabs_Timekeeper_timekeeper                                                            2
LME_HP_-_Software_Delivery_Repository_Firmware_Pack_for_ProLiant_-_6Server_-_Current      1163
LME_HP_-_Software_Delivery_Repository_Scripting_Took_Kit_-_6Server                          17
LME_HP_-_Software_Delivery_Repository_Service_Pack_for_ProLiant_-_6Server_-_Current       1915
LME_HP_-_Software_Delivery_Repository_Smart_Update_Manager_-_6Server                        30
LME_LME_Custom_Product_Mellanox_OFED                                                       114
LME_LME_Custom_Product_OMD_RPMS                                                             14
LME_LME_Custom_Product_RPMs                                                                  5
LME_LME_Custom_Product_SNOW_Repository                                                       2
rhel-6-server-optional-rpms                                                            10400+1
rhel-6-server-rpms                                                                    18256+12
rhel-6-server-satellite-tools-6.2-rpms                                                      55
repolist: 58096
""".strip()

# This output occurred in Bugzilla 1929384
YUM_REPOLIST_ZERO = """
repolist: 0
""".strip()

YUM_REPOLIST_MISSING_HEADER = """
Loaded plugins: product-id, rhnplugin, security, subscription-manager
Updating certificate-based repositories.
"""


def test_yum_repolist():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT))
    assert len(repo_list) == 4
    assert repo_list[0] == {"id": "rhel-7-server-rpms/7Server/x86_64",
                            "name": "Red Hat Enterprise Linux",
                            "status": "10415"}
    assert 'rhel-7-server-rpms/7Server/x86_64' in repo_list
    assert repo_list['rhel-7-server-rpms/7Server/x86_64'] == repo_list[0]
    assert repo_list.eus == []


def test_yum_repolist_noplugins():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_NOPLUGINS))
    assert len(repo_list) == 4
    assert repo_list[0] == {"id": "rhel-7-server-rpms/7Server/x86_64",
                            "name": "Red Hat Enterprise Linux",
                            "status": "10415"}
    assert 'rhel-7-server-rpms/7Server/x86_64' in repo_list
    assert repo_list['rhel-7-server-rpms/7Server/x86_64'] == repo_list[0]
    assert repo_list.eus == []


def test_eus():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_EUS))
    assert len(repo_list) == 2
    assert repo_list[0] == {"id": "clone-6u5-server-x86_64",
                            "name": "clone-6u5-server-x86_64",
                            "status": "3,787"}
    assert repo_list.eus == ["6.2.aus"]


def test_rhel_repos():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT))
    assert len(repo_list.rhel_repos) == 4
    assert set(repo_list.rhel_repos) == set(['rhel-7-server-rpms',
                                             'rhel-7-server-satellite-6.1-rpms',
                                             'rhel-7-server-satellite-capsule-6.1-rpms',
                                             'rhel-server-rhscl-7-rpms'])

    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_RHEL8))
    assert len(repo_list.rhel_repos) == 7
    assert 'satellite-tools-6.7-for-rhel-8-x86_64-rpms' in repo_list.rhel_repos


def test_rhel_repos_missing_status():
    with pytest.raises(ParseException) as se:
        YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_MISSING_STATUS))
    assert 'Incorrect line:' in str(se)


def test_rhel_repos_empty():
    with pytest.raises(SkipException) as se:
        YumRepoList(context_wrap(''))
    assert 'No repolist.' in str(se)

    with pytest.raises(SkipException) as se:
        YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_EMPTY))
    assert 'No repolist.' in str(se)


def test_rhel_repos_out_of_date():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_OUT_OF_DATE))
    assert len(repo_list) == 2
    assert set(repo_list.rhel_repos) == set(['rhel-7-server-extras-rpms',
                                             'rhel-7-server-rpms'])


def test_rhel_repos_out_of_date_and_no_match_metadata():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_OUT_OF_DATE_AND_NOT_MATCH_METADATA))
    assert len(repo_list) == 2
    assert 'rhel-7-server-extras-rpms/x86_64' in repo_list
    assert 'rhel-7-server-rpms/7Server/x86_64' in repo_list
    assert "!rhel-7-server-extras-rpms/x86_64" == repo_list['rhel-7-server-extras-rpms/x86_64']['id']
    assert "*rhel-7-server-rpms/7Server/x86_64" == repo_list['rhel-7-server-rpms/7Server/x86_64']['id']


def test_invalid_get_type():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT))
    assert repo_list[YumRepoList] is None


def test_doc_examples():
    env = {
            'repolist': YumRepoList(context_wrap(YUM_REPOLIST_DOC)),
            'repolist_no_reponame': YumRepoList(context_wrap(YUM_REPOLIST_DOC_NO_REPONAME)),
          }
    failed, total = doctest.testmod(yum, globs=env)
    assert failed == 0


def test_repos_without_repo_name():
    repo_list = YumRepoList(context_wrap(YUM_REPO_NO_REPO_NAME))
    assert 7 == len(repo_list)
    assert 2 == len(repo_list.rhel_repos)


def test_repos_without_ends():
    repo_list = YumRepoList(context_wrap(YUM_REPOLIST_CONTENT_MISSING_END))
    assert 1 == len(repo_list)
    assert 0 == len(repo_list.rhel_repos)


def test_repolist_zero():
    with pytest.raises(SkipException) as se:
        YumRepoList(context_wrap(YUM_REPOLIST_ZERO))
    assert 'No repolist.' in str(se)


def test_repolist_missing_header():
    with pytest.raises(ParseException) as se:
        YumRepoList(context_wrap(YUM_REPOLIST_MISSING_HEADER))
    assert 'Failed to parser yum repolist' in str(se)



"""
rhev_data_center - datasource ``rhev_data_center``
==================================================
"""
import json
from re import compile as re_compile
from insights import CommandParser, parser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.rhev_data_center)
class RhevDataCenter(CommandParser):
    """Walk through the `/rhev/data-center` directory of RHEV host and return the
    full path of files not having correct file ownership i.e vdsm:kvm. See the
    `rhev_data_center` Datasource for more info.

    Attributes:
        data(list): A list of the parsed output returned by `rhev_data_center` Datasource.

        incorrect_volume_ownership(list): Volumes attached to the RHEV VMs in
          the Data Center having incorrect file ownership.

    Raises:
        SkipException: If no files are found with incorrect ownership.

    The following are available in ``data`` and ``incorrect_volume_ownership``:

        * ``name``  - file owner
        * ``group`` - file group
        * ``path``  - full path of a file

    Examples:
        >>> assert len(rhev_dc.data) == 4
        >>> assert len(rhev_dc.incorrect_volume_ownership) == 1
        >>> assert rhev_dc.incorrect_volume_ownership[0]['path'] == '/rhev/data-center/mnt/host1.example.com:_nfsshare_data/a384bf5d-db92-421e-926d-bfb99a6b4b28/images/b7d6cc07-d1f1-44b3-b3c0-7067ec7056a3/4d6e5dea-995f-4a4e-b487-0f70361f6137'
        >>> assert rhev_dc.incorrect_volume_ownership[0]['name'] == 'root'
        >>> assert rhev_dc.incorrect_volume_ownership[0]['group'] == 'root'

    """
    def parse_content(self, content):
        if not content:
            raise SkipException('No files found with incorrect ownership.')
        self.data = json.loads(''.join(content))

        # Full path of volumes attached to the RHEV VMs in the Data Center not having correct file ownership.
        _vol_path_pattern = re_compile("/images/[0-9,a-z,-]*/[0-9,a-z,-]*")
        self.incorrect_volume_ownership = [each for each in self.data if _vol_path_pattern.search(each['path'])]



import doctest

from insights.parsers import ls_ocp_cni_openshift_sdn
from insights.parsers.ls_ocp_cni_openshift_sdn import LsOcpCniOpenshiftSdn
from insights.tests import context_wrap

LS_CNI_OPENSHIFT_SDN = """
total 52
-rw-r--r--. 1 root root 64 Aug  5 23:26 10.130.0.102
-rw-r--r--. 1 root root 64 Aug  5 23:26 10.130.0.103
-rw-r--r--. 1 root root 64 Aug  6 22:52 10.130.0.116
-rw-r--r--. 1 root root 64 Aug  6 22:52 10.130.0.117
-rw-r--r--. 1 root root 64 Aug  5 06:59 10.130.0.15
-rw-r--r--. 1 root root 64 Aug  5 07:02 10.130.0.20
-rw-r--r--. 1 root root 12 Aug  6 22:52 last_reserved_ip.0
""".strip()


def test_ls_ocp_cni_openshift_sdn():
    ls_ocp_cni_openshift_sdn = LsOcpCniOpenshiftSdn(
        context_wrap(LS_CNI_OPENSHIFT_SDN, path='insights_commands/ls_-l_.var.lib.cni.networks.openshift-sdn'))
    assert len(ls_ocp_cni_openshift_sdn.files_of("/var/lib/cni/networks/openshift-sdn")) == 7
    assert ls_ocp_cni_openshift_sdn.files_of("/var/lib/cni/networks/openshift-sdn") == ['10.130.0.102', '10.130.0.103',
                                                                                        '10.130.0.116', '10.130.0.117',
                                                                                        '10.130.0.15', '10.130.0.20',
                                                                                        'last_reserved_ip.0']


def test_ls_ocp_cni_openshift_sdn_doc_examples():
    env = {
        'ls_ocp_cni_openshift_sdn': LsOcpCniOpenshiftSdn(
            context_wrap(LS_CNI_OPENSHIFT_SDN, path='insights_commands/ls_-l_.var.lib.cni.networks.openshift-sdn')),
    }
    failed, total = doctest.testmod(ls_ocp_cni_openshift_sdn, globs=env)
    assert failed == 0



from datetime import datetime
import pytest

from insights.parsers.vdsm_log import VDSMLog, VDSMImportLog
from insights.tests import context_wrap


# This log commonly has \n in it - hence r""
VDSM_VER_3_LOG = r"""
Thread-60::DEBUG::2015-05-04 00:01:07,490::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\n1+0 records out\n4096 bytes (4.1 kB) copied, 0.000147196 s, 27.8 MB/s\n'; <rc> = 0
Thread-64::DEBUG::2015-05-04 00:01:07,894::blockSD::600::Storage.Misc.excCmd::(getReadDelay) '/bin/dd if=/dev/fe4d8910-ac67-4307-a3e7-be8a56d1c559/metadata iflag=direct of=/dev/null bs=4096 count=1' (cwd None)
Thread-64::DEBUG::2015-05-04 00:01:07,909::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\n1+0 records out\n4096 bytes (4.1 kB) copied, 0.00020047 s, 20.4 MB/s\n'; <rc> = 0
Thread-63::DEBUG::2015-05-04 00:01:08,050::blockSD::600::Storage.Misc.excCmd::(getReadDelay) '/bin/dd if=/dev/b0c17a6d-c5a5-4646-b4b5-edd85bc658db/metadata iflag=direct of=/dev/null bs=4096 count=1' (cwd None)
Thread-63::DEBUG::2015-05-04 00:01:08,061::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\n1+0 records out\n4096 bytes (4.1 kB) copied, 0.000632011 s, 6.5 MB/s\n'; <rc> = 0
VM Channels Listener::DEBUG::2015-05-04 00:01:10,830::vmChannels::95::vds::(_handle_timeouts) Timeout on fileno 47.
Thread-13::DEBUG::2015-05-04 00:01:15,853::task::595::TaskManager.Task::(_updateState) Task=`57faf240-0efe-44e2-8485-ffa182bbc9dd`::moving from state init -> state preparing
Thread-13::INFO::2015-05-04 00:01:15,854::logUtils::44::dispatcher::(wrapper) Run and protect: repoStats(options=None)
Thread-13::INFO::2015-05-04 00:01:15,854::logUtils::47::dispatcher::(wrapper) Run and protect: repoStats, Return response: {u'cf7dab23-6b5b-45f4-9e27-ab06fbc01759': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000153278', 'lastCheck': '8.8', 'valid': True}, u'5a30691d-4fae-4023-ae96-50704f6b253c': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000147196', 'lastCheck': '8.4', 'valid': True}, u'b0c17a6d-c5a5-4646-b4b5-edd85bc658db': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000632011', 'lastCheck': '7.8', 'valid': True}, u'fe4d8910-ac67-4307-a3e7-be8a56d1c559': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.00020047', 'lastCheck': '7.9', 'valid': True}, u'e70cce65-0d02-4da4-8781-6aeeef5c86ff': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000208457', 'lastCheck': '9.6', 'valid': True}, u'c3a10b1e-574e-4edd-ad00-a59cacc705b5': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000201237', 'lastCheck': '8.8', 'valid': True}}
"""


def test_vdsm_version_3_log():
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_3_LOG))
    assert "VM Channels Listener" in vdsm_log
    test_newline_handling = vdsm_log.get('SUCCESS')
    assert len(test_newline_handling) == 3
    assert test_newline_handling[0]['raw_message'] == "Thread-60::DEBUG::2015-05-04 00:01:07,490::blockSD::600::Storage.Misc.excCmd::(getReadDelay) SUCCESS: <err> = '1+0 records in\\n1+0 records out\\n4096 bytes (4.1 kB) copied, 0.000147196 s, 27.8 MB/s\\n'; <rc> = 0"
    parsed_newlines = list(vdsm_log.parse_lines(test_newline_handling))
    assert parsed_newlines[0] == {
        'thread': "Thread-60",
        'level': "DEBUG",
        'asctime': datetime(2015, 5, 4, 0, 1, 7, 490000),
        'module': "blockSD",
        'line': "600",
        'logname': "Storage.Misc.excCmd",
        'message': "SUCCESS: <err> = '1+0 records in\\n1+0 records out\\n4096 bytes (4.1 kB) copied, 0.000147196 s, 27.8 MB/s\\n'; <rc> = 0"
    }
    # test get_after()
    assert len(list(vdsm_log.get_after(datetime(2015, 5, 4, 0, 1, 10)))) == 4
    assert len(list(vdsm_log.get_after(datetime(2015, 5, 4, 0, 1, 10), 'dispatcher'))) == 2


BAD_VDSM_VER_3_LOG = r"""
Thread-11::INFO::2015-05-03 00:01:15,853
Thread-12::INFO::2015-05-04 00:01:15,854::
Thread-13::INFO::2015-05-04 00:01:15,854::logUtils::47::dispatcher::(wrapper) Run and protect: repoStats, Return response: {u'cf7dab23-6b5b-45f4-9e27-ab06fbc01759': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000153278', 'lastCheck': '8.8', 'valid': True}, u'5a30691d-4fae-4023-ae96-50704f6b253c': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000147196', 'lastCheck': '8.4', 'valid': True}, u'b0c17a6d-c5a5-4646-b4b5-edd85bc658db': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000632011', 'lastCheck': '7.8', 'valid': True}, u'fe4d8910-ac67-4307-a3e7-be8a56d1c559': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.00020047', 'lastCheck': '7.9', 'valid': True}, u'e70cce65-0d02-4da4-8781-6aeeef5c86ff': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000208457', 'lastCheck': '9.6', 'valid': True}, u'c3a10b1e-574e-4edd-ad00-a59cacc705b5': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000201237', 'lastCheck': '8.8', 'valid': True}}
FATAL ERROR
"""


def test_bad_vdsm_log_parsing():
    vdsm_log = VDSMLog(context_wrap(BAD_VDSM_VER_3_LOG))
    parsed = list(vdsm_log.parse_lines(vdsm_log.lines))
    assert len(parsed) == 4
    assert parsed[0] == {
        'thread': 'Thread-11',
        'level': 'INFO',
        'asctime': datetime(2015, 5, 3, 0, 1, 15, 853000),
    }
    assert parsed[1] == {
        'thread': 'Thread-12',
        'level': 'INFO',
        'asctime': datetime(2015, 5, 4, 0, 1, 15, 854000),
    }
    assert parsed[2] == {
        'thread': 'Thread-13',
        'level': 'INFO',
        'asctime': datetime(2015, 5, 4, 0, 1, 15, 854000),
        'module': 'logUtils',
        'line': '47',
        'logname': 'dispatcher',
        'message': "Run and protect: repoStats, Return response: {u'cf7dab23-6b5b-45f4-9e27-ab06fbc01759': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000153278', 'lastCheck': '8.8', 'valid': True}, u'5a30691d-4fae-4023-ae96-50704f6b253c': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000147196', 'lastCheck': '8.4', 'valid': True}, u'b0c17a6d-c5a5-4646-b4b5-edd85bc658db': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000632011', 'lastCheck': '7.8', 'valid': True}, u'fe4d8910-ac67-4307-a3e7-be8a56d1c559': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.00020047', 'lastCheck': '7.9', 'valid': True}, u'e70cce65-0d02-4da4-8781-6aeeef5c86ff': {'code': 0, 'version': 3, 'acquired': True, 'delay': '0.000208457', 'lastCheck': '9.6', 'valid': True}, u'c3a10b1e-574e-4edd-ad00-a59cacc705b5': {'code': 0, 'version': 0, 'acquired': True, 'delay': '0.000201237', 'lastCheck': '8.8', 'valid': True}}"
    }
    assert parsed[3] == {
        'thread': 'FATAL ERROR'
    }


VDSM_VER_4_LOG_1 = r"""
2017-04-18 13:56:28,096+0200 ERROR (vm/27f1a8d4) [virt.vm] (vmId='27f1a8d4-1f81-49e6-bd83-4ecb0a462b54') The vm start process failed (vm:617)
Traceback (most recent call last):
  File "/usr/share/vdsm/virt/vm.py", line 553, in _startUnderlyingVm
    self._run()
  File "/usr/share/vdsm/virt/vm.py", line 2006, in _run
    self._connection.createXML(domxml, flags),
  File "/usr/lib/python2.7/site-packages/vdsm/libvirtconnection.py", line 123, in wrapper
    ret = f(*args, **kwargs)
  File "/usr/lib/python2.7/site-packages/vdsm/utils.py", line 941, in wrapper
    return func(inst, *args, **kwargs)
  File "/usr/lib64/python2.7/site-packages/libvirt.py", line 3777, in createXML
    if ret is None:raise libvirtError('virDomainCreateXML() failed', conn=self)
libvirtError: internal error: Attempted double use of PCI slot 0000:00:01.0 (may need "multifunction='on'" for device on function 0)
2017-04-18 13:56:28,100+0200 INFO  (vm/27f1a8d4) [virt.vm] (vmId='27f1a8d4-1f81-49e6-bd83-4ecb0a462b54') Changed state to Down: internal error: Attempted double use of PCI slot 0000:00:01.0 (may need "multifunction='on'" for device on function 0) (code=1) (vm:1207)
2017-04-18 13:56:28,101+0200 INFO  (vm/27f1a8d4) [virt.vm] (vmId='27f1a8d4-1f81-49e6-bd83-4ecb0a462b54') Stopping connection (guestagent:430)
2017-04-18 13:56:29,121+0200 INFO  (jsonrpc/1) [vdsm.api] START destroy(gracefulAttempts=1) (api:39)
2017-04-18 13:56:29,122+0200 INFO  (jsonrpc/1) [virt.vm] (vmId='27f1a8d4-1f81-49e6-bd83-4ecb0a462b54') Release VM resources (vm:4199)
"""

VDSM_VER_4_LOG_2 = r"""
2017-05-22 11:50:45,363+0530 ERROR (vm/1b8dc1a2) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') The vm start process failed (vm:617)
Traceback (most recent call last):
  File "/usr/share/vdsm/virt/vm.py", line 553, in _startUnderlyingVm
    self._run()
  File "/usr/share/vdsm/virt/vm.py", line 2006, in _run
    self._connection.createXML(domxml, flags),
  File "/usr/lib/python2.7/site-packages/vdsm/libvirtconnection.py", line 123, in wrapper
    ret = f(*args, **kwargs)
  File "/usr/lib/python2.7/site-packages/vdsm/utils.py", line 941, in wrapperrr
    return func(inst, *args, **kwargs)
  File "/usr/lib64/python2.7/site-packages/libvirt.py", line 3777, in createXML
    if ret is None:raise libvirtError('virDomainCreateXML() failed', conn=self)
libvirtError: internal error: Attempted double use of PCI slot 0000:00:01.0 (may need "multifunction='on'" for device on function 0)
2017-05-22 11:50:45,365+0530 INFO  (vm/1b8dc1a2) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') Changed state to Down: internal error: Attempted double use of PCI slot 0000:00:01.0 (may need "multifunction='on'" for device on function 0) (code=1) (vm:1207)
2017-05-22 11:50:45,365+0530 INFO  (vm/1b8dc1a2) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') Stopping connection (guestagent:430)
2017-05-22 11:50:46,061+0530 INFO  (jsonrpc/3) [dispatcher] Run and protect: getSpmStatus(spUUID=u'5a9ca74c-fbc1-4844-92db-18086447c491', options=None) (logUtils:51)
2017-05-22 11:50:46,073+0530 INFO  (jsonrpc/3) [dispatcher] Run and protect: getSpmStatus, Return response: {'spm_st': {'spmId': 2, 'spmStatus': 'SPM', 'spmLver': 40L}} (logUtils:54)
2017-05-22 11:50:46,073+0530 INFO  (jsonrpc/3) [jsonrpc.JsonRpcServer] RPC call StoragePool.getSpmStatus succeeded in 0.01 seconds (__init__:515)
2017-05-22 11:50:46,079+0530 INFO  (jsonrpc/1) [vdsm.api] START destroy(gracefulAttempts=1) (api:39)
2017-05-22 11:50:46,080+0530 INFO  (jsonrpc/1) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') Release VM resources (vm:4199)
2017-05-22 11:50:46,080+0530 WARN  (jsonrpc/1) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') trying to set state to Powering down when already Down (vm:352)
2017-05-22 11:50:46,080+0530 INFO  (jsonrpc/1) [virt.vm] (vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') Stopping connection (guestagent:430)
"""

VDSM_VER_4_LOG_3 = r"""
2017-04-18 13:56:33,718+0200 INFO  (jsonrpc/3) [jsonrpc.JsonRpcServer] RPC call Host.getStats succeeded in 0.03 seconds (__init__:515)
2017-04-18 13:56:35,208+0200 INFO  (jsonrpc/5) [jsonrpc.JsonRpcServer] RPC call Host.getAllVmStats succeeded in 0.01 seconds (__init__:515)
2017-04-18 13:56:38,297+0200 INFO  (itmap/0) [IOProcessClient] Starting client ioprocess-29890 (__init__:325)
2017-04-18 13:56:38,315+0200 INFO  (itmap/1) [IOProcessClient] Starting client ioprocess-29891 (__init__:325)
2017-04-18 13:56:38,330+0200 INFO  (ioprocess communication (16074)) [IOProcess] Starting ioprocess (__init__:447)
2017-04-18 13:56:38,335+0200 INFO  (itmap/2) [IOProcessClient] Starting client ioprocess-29892 (__init__:325)
2017-04-18 13:56:38,354+0200 INFO  (jsonrpc/2) [dispatcher] Run and protect: getSpmStatus(spUUID=u'00000002-0002-0002-0002-00000000024f', options=None) (logUtils:51)
2017-04-18 13:56:38,357+0200 INFO  (ioprocess communication (16082)) [IOProcess] Starting ioprocess (__init__:447)
2017-04-18 13:56:38,360+0200 INFO  (itmap/3) [IOProcessClient] Starting client ioprocess-29893 (__init__:325)
2017-04-18 13:56:38,374+0200 INFO  (jsonrpc/2) [dispatcher] Run and protect: getSpmStatus, Return response: {'spm_st': {'spmId': 1, 'spmStatus': 'SPM', 'spmLver': 3L}} (logUtils:54)
2017-04-18 13:56:38,376+0200 INFO  (jsonrpc/2) [jsonrpc.JsonRpcServer] RPC call StoragePool.getSpmStatus succeeded in 0.02 seconds (__init__:515)
2017-04-18 13:56:38,382+0200 INFO  (ioprocess communication (16090)) [IOProcess] Starting ioprocess (__init__:447)
2017-04-18 13:56:38,386+0200 INFO  (itmap/4) [IOProcessClient] Starting client ioprocess-29894 (__init__:325)
2017-04-18 13:56:38,401+0200 INFO  (ioprocess communication (16097)) [IOProcess] Starting ioprocess (__init__:447)
2017-04-18 13:56:38,405+0200 INFO  (ioprocess communication (16104)) [IOProcess] Starting ioprocess (__init__:447)
2017-04-18 13:56:38,427+0200 INFO  (monitor/0c78b4d) [storage.StorageDomain] Resource namespace 01_img_0c78b4d6-ba00-4d3e-9f9f-65c7d5899d71 already registered (sd:732)
"""

VDSM_VER_4_LOG_4 = r"""
2017-04-18 14:00:00,000+0200 INFO  (jsonrpc/2) [jsonrpc.JsonRpcServer] RPC call Host.getStats succeeded in 0.02 seconds (__init__:515)
2017-04-18 14:00:01,807+0200 INFO  (Reactor thread) [ProtocolDetector.AcceptorImpl] Accepted connection from ::ffff:10.34.60.219:49213 (protocoldetector:72)
2017-04-18 14:00:01,808+0200 ERROR (Reactor thread) [ProtocolDetector.SSLHandshakeDispatcher] Error during handshake: unexpected eof (m2cutils:304)
2017-04-18 14:00:03,304+0200 INFO  (jsonrpc/6) [jsonrpc.JsonRpcServer] RPC call Host.getAllVmStats succeeded in 0.00 seconds (__init__:515)
2017-04-18 14:00:05,870+0200 INFO  (jsonrpc/7) [dispatcher] Run and protect: getSpmStatus(spUUID=u'00000002-0002-0002-0002-00000000024f', options=None) (logUtils:51)
"""

VDSM_VER_4_LOG_5 = r"""
2018-01-31 09:20:02,658+0530 DEBUG (check/loop) [storage.check] FINISH check u'/rhev/data-center/mnt/rhevm.example.com:_home_exports_iso/be94b389-000f-4487-baf4-35ded403e579/dom_md/metadata' (rc=0, elapsed=0.05) (check:328)
2018-01-31 09:20:02,955+0530 DEBUG (mailbox-spm) [storage.Misc.excCmd] /usr/bin/taskset --cpu-list 0-15 dd if=/rhev/data-center/5a5e5d2a-0076-00b1-00b8-00000000006f/mastersd/dom_md/inbox iflag=direct,fullblock count=1 bs=1024000 (cwd None) (commands:69)
2018-01-31 09:20:03,006+0530 DEBUG (mailbox-spm) [storage.Misc.excCmd] SUCCESS: <err> = '1+0 records in\n1+0 records out\n1024000 bytes (1.0 MB) copied, 0.0350045 s, 29.3 MB/s\n'; <rc> = 0 (commands:93)
2018-01-31 09:20:05,016+0530 DEBUG (mailbox-spm) [storage.Misc.excCmd] /usr/bin/taskset --cpu-list 0-15 dd if=/rhev/data-center/5a5e5d2a-0076-00b1-00b8-00000000006f/mastersd/dom_md/inbox iflag=direct,fullblock count=1 bs=1024000 (cwd None) (commands:69)
2018-01-31 09:20:05,040+0530 DEBUG (mailbox-spm) [storage.Misc.excCmd] SUCCESS: <err> = '1+0 records in\n1+0 records out\n1024000 bytes (1.0 MB) copied, 0.0171592 s, 59.7 MB/s\n'; <rc> = 0 (commands:93)
"""


def test_vdsm_version_4_log():
    # Check lines with level 'ERROR'.
    # (psachin): This will NOT parse Python Traceback
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_4_LOG_1))
    assert 'libvirtError' in vdsm_log
    lines_with_error = vdsm_log.get('The vm start process failed')
    parsed = list(vdsm_log.parse_lines(lines_with_error))
    assert parsed[0] == {
        'thread': 'vm/27f1a8d4',
        'level': 'ERROR',
        'logname': 'virt.vm',
        'module': 'vm',
        'asctime': datetime(2017, 4, 18, 13, 56, 28, 96000),
        'lineno': '617',
        'message': "(vmId='27f1a8d4-1f81-49e6-bd83-4ecb0a462b54') The vm start process failed"  # noqa
    }

    # Looks for traceback
    assert 'Traceback' in vdsm_log

    # This will not work(== 0) as get_after() don't work for milliseconds
    after_error_logs = vdsm_log.get_after(parsed[0]['asctime'], 'The vm start process failed')
    assert len(list(after_error_logs)) == 0

    # Try to see if the INFO logs with same thread has messages with content '..PCI slot' and 'Stopping connection'
    info_lines = list(vdsm_log.parse_lines(vdsm_log.get(parsed[0]['thread'])))
    assert len(info_lines) == 3
    for line in info_lines:
        if 'Attempted double use of PCI slot' in line['message'] and line['level'] == 'INFO':
            assert line['logname'] == 'virt.vm'
        if 'Stopping connection' in line['message'] and line['level'] == 'INFO':
            assert line['module'] == 'guestagent'

    # Another logs with line having 'ERROR' but with different
    # thread_name & datetime,
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_4_LOG_2))
    assert 'libvirtError' in vdsm_log
    lines_with_error = vdsm_log.get('ERROR')
    parsed = list(vdsm_log.parse_lines(lines_with_error))
    assert parsed[0] == {
        'thread': 'vm/1b8dc1a2',
        'level': 'ERROR',
        'logname': 'virt.vm',
        'module': 'vm',
        'asctime': datetime(2017, 5, 22, 11, 50, 45, 363000),
        'lineno': '617',
        'message': "(vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') The vm start process failed"  # noqa
    }

    # Parse line with level 'WARN'
    assert 'WARN' in vdsm_log
    lines_with_warn = vdsm_log.get('WARN')
    parsed = list(vdsm_log.parse_lines(lines_with_warn))
    assert parsed[0] == {
        'asctime': datetime(2017, 5, 22, 11, 50, 46, 80000),
        'level': 'WARN',
        'lineno': '352',
        'message': "(vmId='1b8dc1a2-acd3-4a57-9a2b-23f12ae52e38') trying to set state to Powering down when already Down",  # noqa
        'module': 'vm',
        'logname': 'virt.vm',
        'thread': 'jsonrpc/1'
    }

    # Logs with lines having level 'INFO'
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_4_LOG_3))
    lines_with_info = vdsm_log.get('INFO')
    parsed = list(vdsm_log.parse_lines(lines_with_info))
    assert len(parsed) == 16
    assert parsed[15] == {
        'asctime': datetime(2017, 4, 18, 13, 56, 38, 427000),
        'level': 'INFO',
        'lineno': '732',
        'message': 'Resource namespace 01_img_0c78b4d6-ba00-4d3e-9f9f-65c7d5899d71 already registered',  # noqa
        'module': 'sd',
        'logname': 'storage.StorageDomain',
        'thread': 'monitor/0c78b4d',
    }

    lines_with_spm_st = vdsm_log.get('spm_st')
    parsed = list(vdsm_log.parse_lines(lines_with_spm_st))
    assert parsed[0] == {
        'asctime': datetime(2017, 4, 18, 13, 56, 38, 374000),
        'level': 'INFO',
        'lineno': '54',
        'message': "Run and protect: getSpmStatus, Return response: {'spm_st': {'spmId': 1, 'spmStatus': 'SPM', 'spmLver': 3L}}",  # noqa
        'module': 'logUtils',
        'logname': 'dispatcher',
        'thread': 'jsonrpc/2'
    }

    # Logs with line having ERROR with different fields than
    # VDSM_LOG_1 & VDSM_LOG_2 altogether
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_4_LOG_4))
    assert 'ERROR' in vdsm_log
    lines_with_error = vdsm_log.get('ERROR')
    parsed = list(vdsm_log.parse_lines(lines_with_error))
    assert parsed[0] == {
        'asctime': datetime(2017, 4, 18, 14, 0, 1, 808000),
        'level': 'ERROR',
        'lineno': '304',
        'message': 'Error during handshake: unexpected eof',
        'module': 'm2cutils',
        'logname': 'ProtocolDetector.SSLHandshakeDispatcher',
        'thread': 'Reactor thread'
    }

    # Check DEBUG logs
    vdsm_log = VDSMLog(context_wrap(VDSM_VER_4_LOG_5))

    assert 'DEBUG' in vdsm_log
    lines_with_debug = vdsm_log.get('DEBUG')
    parsed = list(vdsm_log.parse_lines(lines_with_debug))
    assert len(parsed) == 5

    lines_with_storage_check = vdsm_log.get('storage.check')
    parsed = list(vdsm_log.parse_lines(lines_with_storage_check))
    assert len(parsed) == 1
    assert parsed[0] == {
        'asctime': datetime(2018, 1, 31, 9, 20, 2, 658000),
        'level': 'DEBUG',
        'lineno': '328',
        'message': "FINISH check u'/rhev/data-center/mnt/rhevm.example.com:_home_exports_iso/be94b389-000f-4487-baf4-35ded403e579/dom_md/metadata' (rc=0, elapsed=0.05)",
        'module': 'check',
        'logname': 'storage.check',
        'thread': 'check/loop'
    }


VDSM_IMPORT_LOGS = """
[    0.2] preparing for copy
[    0.2] Copying disk 1/1 to /rhev/data-center/958ca292-9126-41f1-b7eb-2f9931fea2fb/f524d2ba-155a-45c8-b3ab-9e18539d3ef2/images/502f5598-335d-4023-8c01-635d86d129a3/d4b140c8-9cd5-4ad4-bb04-63aebeda808b
    (0/100%)
    (0/100%)
    (0/100%)
    (0/100%)
    (0/100%)
    (0/100%)
    (1/100%)
    (1/100%)
    (1/100%)
    (1/100%)
    (1/100%)
    (2/100%)
    (2/100%)
    (2/100%)
    (2/100%)
[  546.9] Finishing off
""".strip()

VDSM_IMPORT_FAILURE_LOGS = """
[    0.0] >>> source, dest, and storage-type have different lengths
[    0.2] preparing for copy
[  546.9] Finishing off
""".strip()


def test_vdsm_import_log():
    vdsm_import1 = context_wrap(VDSM_IMPORT_LOGS, path='var/log/vdsm/import/import-1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0-20180130T154807.log')
    result1 = VDSMImportLog(vdsm_import1)
    log1 = result1.get('preparing for copy')
    assert len(log1) == 1
    assert log1[0].get('raw_message') == '[    0.2] preparing for copy'
    assert result1.vm_uuid == '1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0'
    assert len(list(result1.get_after(546.9))) == 1

    # Invalid datetime in file name
    vdsm_import2 = context_wrap(VDSM_IMPORT_LOGS, path='var/log/vdsm/import/import-1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0-201801304807.log')
    result2 = VDSMImportLog(vdsm_import2)
    assert result2.file_datetime is None
    assert result2.vm_uuid == '1f9efdf5-2584-4a2a-8f85-c3b6f5dac4e0'
    log2 = result2.get('Finishing off')
    assert len(log2) == 1


def test_vdsm_import_log2():
    vdsm_import = context_wrap(VDSM_IMPORT_FAILURE_LOGS, path='var/log/vdsm/import/import-3a51988d-579b-42d5-9af5-d48decc85fe4-20180201T082836.log')
    output = VDSMImportLog(vdsm_import)
    assert output.vm_uuid == '3a51988d-579b-42d5-9af5-d48decc85fe4'
    assert output.file_datetime == datetime(2018, 2, 1, 8, 28, 36)
    assert len(list(output.get_after(0.2))) == 2
    assert len(list(output.get_after(0.2, s='Finishing'))) == 1

    with pytest.raises(TypeError) as excinfo:
        list(output.get_after('0.2'))
    assert "get_after needs a float type timestamp, but get '0.2'" in str(excinfo)



"""
NumericUserGroupName - command ``/usr/bin/grep -c '^[[:digit:]]' /etc/passwd /etc/group``
=========================================================================================

This parser reads the output of ``/usr/bin/grep -c '^[[:digit:]]' /etc/passwd /etc/group``
and detects whether there are any user or group names that begin with a digit.

The grep command returns the number of matches for each file. It matches for user and
group names that begin with a number.

The answers are available through attributes - boolean ``has_numeric_user_group`` and
numeric ``nr_numeric_user`` and ``nr_numeric_group``.

Names starting with a digit can be handled incorrectly by some software. Purely numeric
user names can be handled incorrectly by some software and requires special care when
used with utilities that accept both user/group IDs and names (like chown, chmod). In
case such names exist on the system, it is advisable to review whether the used software
(namely 3rd party software and shell scripts) handles the names correctly.

https://access.redhat.com/solutions/3103631

Sample input::

    /etc/passwd:3
    /etc/group:0

Examples:

    >>> shared[NumericUserGroupName].has_numeric_user_or_group
    True
    >>> shared[NumericUserGroupName].nr_numeric_user
    3
    >>> shared[NumericUserGroupName].nr_numeric_group
    0
"""

from .. import parser, CommandParser
from . import ParseException
from insights.specs import Specs


@parser(Specs.numeric_user_group_name)
class NumericUserGroupName(CommandParser):
    """
    Reports whether there is a user or group with a name that starts with a digit.
    """
    def parse_content(self, content):
        self.nr_numeric_user = None
        self.nr_numeric_group = None
        for line in content:
            line_split = line.split(":")
            if len(line_split) != 2:
                continue
            try:
                n = int(line_split[1])
            except ValueError:
                continue
            if '/etc/passwd' == line_split[0]:
                self.nr_numeric_user = n
            if '/etc/group' == line_split[0]:
                self.nr_numeric_group = n

        # The grep command should report on both files unless a file is missing or something is very
        # wrong.
        if None is self.nr_numeric_user or None is self.nr_numeric_group:
            raise ParseException("Unexpected grep output or grep not found.")

        self.has_numeric_user_or_group = bool(self.nr_numeric_user or self.nr_numeric_group)



"""
InitScript - files ``/etc/rc.d/init.d/*``
=========================================

InitScript is a parser for the initscripts in ``/etc/rc.d/init.d``.

Because this parser read multiple files, the initscripts are stored as a list
within the parser and need to be iterated through in order to find specific
initscripts.

Examples:

    >>> for initscript in shared[InitScript]: # Parser contains list of all initscripts
    ...     print "Name:", initscript.file_name
    ...
    Name: netconsole
    Name: rhnsd

"""

from . import ParseException
from .. import Parser, parser
import re
from insights.specs import Specs

CHKCONFIG_REGEX = re.compile(r"^#\s+chkconfig:\s+.*$")
LSB_REGEX = re.compile(r"^#\s+Provides:\s+.*$")

SHEBANG_REGEX = re.compile(r"^#!\s*/.*$")
COMMENT_REGEX = re.compile(r"^\s*#.*$")
STARTSTOPSTATUS_REGEX = re.compile(r"\b(start|stop|status)\b")


class EmptyFileException(ParseException):
    pass


class NotInitscriptException(ParseException):
    pass


@parser(Specs.initscript)
class InitScript(Parser):
    """
    Parse initscript files. Each item is a dictionary with following fields:

    Attributes:
        file_name (str): initscript name
        file_path (str): initscript path without leading '/'
        file_content (list): initscript content, line by line

    Because some files may not be real initscripts, to determine whether a file
    in ``etc/rc.d/init.d/`` is an initscript, the parser checks for
    ``# chkconfig: <values>`` or ``# Provides: <names>`` strings in the script.
    If that matches, then it assumes it is an initscript.

    Otherwise, it tries to find out if it is by searching for

    * shebang (e.g. ``#!/bin/bash``) on first line
    * ``start``/``stop``/``status`` tokens in non-commented out lines

    If 3 or more items are found (half the items searched for + 1), called
    *confidence* in the code (e.g.  shebang + start + stop), then we assume it
    is an initscript.

    Otherwise the parser raises a ``ParseException``.
    """

    def parse_content(self, content):
        """
        Raises:
            EmptyFileException: Raised if file is empty.
            NotInitscriptException: Raised if likely not an initscript.
        """
        self.file_content = content

        # If we find 'chkconfig: XYZ' or 'Provides: NAME', assume we have an
        # initscript
        for line in content:
            if CHKCONFIG_REGEX.match(line) or LSB_REGEX.match(line):
                return

        # Otherwise, check for hints (shebang + presence of start/stop/status
        # keywords)
        #
        # Allocate 1 point for each item found. And declare as valid if
        # confidence >= 3 (half of the number of items + 1).
        confidence = 0
        try:
            line = content[0]
            if SHEBANG_REGEX.match(line):
                confidence += 1
        except IndexError:
            raise EmptyFileException(self.file_path)

        key = {'start': 0, 'stop': 0, 'status': 0}
        for line in content:
            if COMMENT_REGEX.match(line):
                continue
            m = STARTSTOPSTATUS_REGEX.search(line)
            if m:
                key[m.group(1)] += 1
        confidence += len([v for v in key.values() if v != 0])

        if confidence < 3:
            raise NotInitscriptException("path: %s, confidence: %d" % (self.file_path, confidence))



import doctest
from insights.parsers.mount import Mount, ProcMounts, MountInfo
from insights.parsers.mount import MountOpts, MountAddtlInfo
from insights.combiners.mounts import Mounts
from insights.combiners import mounts as module_mounts
from insights.tests import context_wrap


MOUNT_DATA = """
/dev/mapper/vgsys-root on / type ext4 (rw)
proc on /proc type proc (rw)
sysfs on /sys type sysfs (rw)
devpts on /dev/pts type devpts (rw,gid=5,mode=620)
tmpfs on /dev/shm type tmpfs (rw,noexec,nosuid,nodev)
/dev/sda1 on /boot type ext4 (rw)
/dev/mapper/vgsys-diskdump on /diskdump type ext4 (rw)
/dev/mapper/vgsys-tmp on /tmp type ext4 (rw)
/dev/mapper/vgsys-var on /var type ext4 (rw)
/dev/mapper/vgsys-var_log on /var/log type ext4 (rw) [cluster7:lv_gfs]
/dev/mapper/vgsys-var_tmp on /var/tmp type ext4 (rw) [VMware Tools]
none on /proc/sys/fs/binfmt_misc type binfmt_misc (rw)
sunrpc on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw)
hostname1:/shared_dir1 on /shared/dir1 type nfs (rw,nosuid,sync,hard,intr,bg,tcp,timeo=600,rsize=32768,nolock,wsize=32768,noacl,vers=4,addr=10.72.36.91,clientaddr=10.72.32.16)
hostname2:/shared/some/dir2 on /shared/dir2 type nfs (rw,nosuid,sync,hard,intr,bg,tcp,timeo=600,rsize=32768,nolock,wsize=32768,noacl,addr=10.73.32.202)
""".strip()

PROCMOUNTS_DATA = """
rootfs / rootfs rw 0 0
proc /proc proc rw,relatime 0 0
sysfs /sys sysfs rw,relatime 0 0
devtmpfs /dev devtmpfs rw,relatime,size=66002212k,nr_inodes=16500553,mode=755 0 0
devpts /dev/pts devpts rw,relatime,gid=5,mode=620,ptmxmode=000 0 0
tmpfs /dev/shm tmpfs rw,nosuid,nodev,noexec,relatime 0 0
/dev/mapper/vgsys-root / ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/proc/bus/usb /proc/bus/usb usbfs rw,relatime 0 0
/dev/sda1 /boot ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/dev/mapper/vgsys-diskdump /diskdump ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/dev/mapper/vgsys-tmp /tmp ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/dev/mapper/vgsys-var /var ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/dev/mapper/vgsys-var_log /var/log ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
/dev/mapper/vgsys-var_tmp /var/tmp ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0
none /proc/sys/fs/binfmt_misc binfmt_misc rw,relatime 0 0
sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
hostname1:/shared_dir1 /shared/dir1 nfs4 rw,sync,nosuid,relatime,vers=4,rsize=32768,wsize=32768,namlen=255,hard,proto=tcp,port=0,timeo=600,retrans=2,sec=sys,clientaddr=10.72.32.16,minorversion=0,local_lock=none,addr=10.72.36.91 0 0
hostname2:/shared/some/dir2 /shared/dir2 nfs rw,sync,nosuid,relatime,vers=3,rsize=32768,wsize=32768,namlen=255,hard,nolock,noacl,proto=tcp,timeo=600,retrans=2,sec=sys,mountaddr=10.73.32.108,mountvers=3,mountport=300,mountproto=tcp,local_lock=all,addr=10.73.32.202 0 0
/etc/auto.misc /misc autofs rw,relatime,fd=7,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect 0 0
-hosts /autofsmp autofs rw,relatime,fd=13,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect 0 0
""".strip()

MOUNTINFO_DATA = """
16 21 0:3 / /proc rw,relatime - proc proc rw
17 21 0:0 / /sys rw,relatime - sysfs sysfs rw
18 21 0:5 / /dev rw,relatime - devtmpfs devtmpfs rw,size=66002212k,nr_inodes=16500553,mode=755
19 18 0:11 / /dev/pts rw,relatime - devpts devpts rw,gid=5,mode=620,ptmxmode=000
20 18 0:16 / /dev/shm rw,nosuid,nodev,noexec,relatime - tmpfs tmpfs rw
21 1 253:1 / / rw,relatime - ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered
22 16 0:15 / /proc/bus/usb rw,relatime - usbfs /proc/bus/usb rw
23 21 8:1 / /boot rw,relatime - ext4 /dev/sda1 rw,barrier=1,stripe=64,data=ordered
25 21 253:32 / /diskdump rw,relatime - ext4 /dev/mapper/vgsys-diskdump rw,barrier=1,stripe=64,data=ordered
33 21 253:33 / /tmp rw,relatime - ext4 /dev/mapper/vgsys-tmp rw,barrier=1,stripe=64,data=ordered
36 21 253:34 / /var rw,relatime - ext4 /dev/mapper/vgsys-var rw,barrier=1,stripe=64,data=ordered
37 36 253:36 / /var/log rw,relatime - ext4 /dev/mapper/vgsys-var_log rw,barrier=1,stripe=64,data=ordered
40 36 253:35 / /var/tmp rw,relatime - ext4 /dev/mapper/vgsys-var_tmp rw,barrier=1,stripe=64,data=ordered
52 16 0:17 / /proc/sys/fs/binfmt_misc rw,relatime - binfmt_misc none rw
53 36 0:18 / /var/lib/nfs/rpc_pipefs rw,relatime - rpc_pipefs sunrpc rw
55 21 0:20 / /shared/dir1 rw,nosuid,relatime - nfs4 hostname1:/shared_dir1 rw,sync,vers=4,rsize=32768,wsize=32768,namlen=255,hard,proto=tcp,port=0,timeo=600,retrans=2,sec=sys,clientaddr=10.72.32.16,minorversion=0,local_lock=none,addr=10.72.36.91
56 21 0:19 / /shared/dir2 rw,nosuid,relatime - nfs hostname2:/shared/some/dir2 rw,sync,vers=3,rsize=32768,wsize=32768,namlen=255,hard,nolock,noacl,proto=tcp,timeo=600,retrans=2,sec=sys,mountaddr=10.73.32.108,mountvers=3,mountport=300,mountproto=tcp,local_lock=all,addr=10.73.32.202
57 21 0:21 / /misc rw,relatime - autofs /etc/auto.misc rw,fd=7,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect
58 21 0:22 / /autofsmp rw,relatime - autofs -hosts rw,fd=13,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect
""".strip()


def test_combiner_mounts_all():
    mount = Mount(context_wrap(MOUNT_DATA))
    procmounts = ProcMounts(context_wrap(PROCMOUNTS_DATA))
    mountinfo = MountInfo(context_wrap(MOUNTINFO_DATA))
    mounts = Mounts(mount, procmounts, mountinfo)
    assert len(mounts) == 19

    assert '/' in mounts
    root_entry = mounts['/']
    assert root_entry.mount_type == 'ext4'
    assert len(root_entry.mount_options) == 5
    assert root_entry.mount_options.data == 'ordered'
    root_entry_opts = MountOpts({'rw': True, 'relatime': True, 'barrier': '1', 'stripe': '64', 'data': 'ordered'})
    for k, v in root_entry_opts.items():
        assert root_entry.mount_options[k] == v
    root_entry_addtlinfo = MountAddtlInfo(dict(fs_freq='0', fs_passno='0',
                mount_id='21', parent_id='1', major_minor='253:1', root='/', optional_fields='',
                mount_clause_binmount='/dev/mapper/vgsys-root on / type ext4 (rw)',
                mount_clause_procmounts='/dev/mapper/vgsys-root / ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0',
                mount_clause_mountinfo='21 1 253:1 / / rw,relatime - ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered'))
    for k, v in root_entry_addtlinfo.items():
        assert root_entry.mount_addtlinfo[k] == v

    assert mounts.search(mount_source="/dev/mapper/vgsys-root")[0]['mount_point'] == '/'

    assert mounts.rows[0] == mounts['/proc']
    assert len([row for row in mounts]) == 19
    assert len(mounts.mount_points) == 19
    assert mounts.get_dir('/var/somedir') == mounts['/var']
    assert mounts.get_dir('not/absolute/path') is None


def test_combiner_wo_mountinfo():
    mount = Mount(context_wrap(MOUNT_DATA))
    procmounts = ProcMounts(context_wrap(PROCMOUNTS_DATA))
    mounts = Mounts(mount, procmounts, None)
    assert len(mounts) == 19

    assert '/' in mounts
    root_entry = mounts['/']
    assert root_entry.mount_type == 'ext4'
    root_entry_opts = MountOpts({'rw': True, 'relatime': True, 'barrier': '1', 'stripe': '64', 'data': 'ordered'})
    for k, v in root_entry_opts.items():
        assert root_entry.mount_options[k] == v
    root_entry_addtlinfo = MountAddtlInfo(dict(fs_freq='0', fs_passno='0',
                mount_clause_binmount='/dev/mapper/vgsys-root on / type ext4 (rw)',
                mount_clause_procmounts='/dev/mapper/vgsys-root / ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0'))
    for k, v in root_entry_addtlinfo.items():
        assert root_entry.mount_addtlinfo[k] == v

    assert mounts.search(mount_source="/dev/mapper/vgsys-root")[0]['mount_point'] == '/'
    assert mounts.rows[0] == mounts['/']


def test_combiner_wo_binmount():
    procmounts = ProcMounts(context_wrap(PROCMOUNTS_DATA))
    mountinfo = MountInfo(context_wrap(MOUNTINFO_DATA))
    mounts = Mounts(None, procmounts, mountinfo)
    assert len(mounts) == 19

    assert '/' in mounts
    root_entry = mounts['/']
    assert root_entry.mount_type == 'ext4'
    root_entry_opts = MountOpts({'rw': True, 'relatime': True, 'barrier': '1', 'stripe': '64', 'data': 'ordered'})
    for k, v in root_entry_opts.items():
        assert root_entry.mount_options[k] == v
    root_entry_addtlinfo = MountAddtlInfo(dict(fs_freq='0', fs_passno='0',
                mount_id='21', parent_id='1', major_minor='253:1', root='/', optional_fields='',
                mount_clause_procmounts='/dev/mapper/vgsys-root / ext4 rw,relatime,barrier=1,stripe=64,data=ordered 0 0',
                mount_clause_mountinfo='21 1 253:1 / / rw,relatime - ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered'))
    for k, v in root_entry_addtlinfo.items():
        assert root_entry.mount_addtlinfo[k] == v

    assert mounts.search(mount_source="/dev/mapper/vgsys-root")[0]['mount_point'] == '/'
    assert mounts.rows[0] == mounts['/proc']


def test_combiner_wo_procmount():
    mount = Mount(context_wrap(MOUNT_DATA))
    mountinfo = MountInfo(context_wrap(MOUNTINFO_DATA))
    mounts = Mounts(mount, None, mountinfo)
    assert len(mounts) == 19

    assert '/' in mounts
    root_entry = mounts['/']
    assert root_entry.mount_type == 'ext4'
    root_entry_opts = MountOpts({'rw': True, 'relatime': True, 'barrier': '1', 'stripe': '64', 'data': 'ordered'})
    for k, v in root_entry_opts.items():
        assert root_entry.mount_options[k] == v
    root_entry_addtlinfo = MountAddtlInfo(dict(
                mount_id='21', parent_id='1', major_minor='253:1', root='/', optional_fields='',
                mount_clause_binmount='/dev/mapper/vgsys-root on / type ext4 (rw)',
                mount_clause_mountinfo='21 1 253:1 / / rw,relatime - ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered'))
    for k, v in root_entry_addtlinfo.items():
        assert root_entry.mount_addtlinfo[k] == v

    assert mounts.search(mount_source="/dev/mapper/vgsys-root")[0]['mount_point'] == '/'
    assert mounts.rows[0] == mounts['/proc']


def test_combiner_only_binmount():
    mount = Mount(context_wrap(MOUNT_DATA))
    mounts = Mounts(mount, None, None)
    assert len(mounts) == 15

    assert '/' in mounts
    root_entry = mounts['/']
    assert root_entry.mount_type == 'ext4'
    root_entry_opts = MountOpts({'rw': True})
    for k, v in root_entry_opts.items():
        assert root_entry.mount_options[k] == v
    root_entry_addtlinfo = MountAddtlInfo(dict(
                mount_clause_binmount='/dev/mapper/vgsys-root on / type ext4 (rw)'))
    for k, v in root_entry_addtlinfo.items():
        assert root_entry.mount_addtlinfo[k] == v

    assert mounts.search(mount_source="/dev/mapper/vgsys-root")[0]['mount_point'] == '/'
    assert mounts.rows[0] == mounts['/']


def test_docs():
    mount = Mount(context_wrap(MOUNT_DATA))
    procmounts = ProcMounts(context_wrap(PROCMOUNTS_DATA))
    mountinfo = MountInfo(context_wrap(MOUNTINFO_DATA))
    mounts = Mounts(mount, procmounts, mountinfo)
    env = {
        'mounts': mounts
    }
    failed, total = doctest.testmod(module_mounts, globs=env)
    assert failed == 0



"""
WcProc1Mountinfo - Command ``/usr/bin/wc -l /proc/1/mountinfo``
===============================================================

Parser for parsing the output of command ``/usr/bin/wc -l /proc/1/mountinfo``.
"""

from insights import Parser, parser
from insights.core.dr import SkipComponent
from insights.parsers import ParseException
from insights.specs import Specs


@parser(Specs.wc_proc_1_mountinfo)
class WcProc1Mountinfo(Parser):
    """
    Provides the line counts of file ``/proc/1/mountinfo`` by parsing the
    output of command ``/usr/bin/wc -l /proc/1/mountinfo``.

    Attributes:
        line_count(int): the line counts of file ``/proc/1/mountinfo``

    Typical content looks like::

        37 /proc/1/mountinfo

    Examples:
        >>> type(wc_info)
        <class 'insights.parsers.wc_proc_1_mountinfo.WcProc1Mountinfo'>
        >>> wc_info.line_count
        37

    Raises:
        SkipComponent: if the command output is empty or missing file
        ParseException: if the command output is unparsable
    """

    def parse_content(self, content):
        if len(content) == 0 or 'No such file or directory' in content[0]:
            raise SkipComponent("Error: ", content[0] if content else 'empty file')
        count_str = content[0].split()[0]
        if not count_str.isdigit():
            raise ParseException("Error: unparsable output from command wc: ", content[0])
        self.line_count = int(count_str)



"""
Custom datasources related to ``corosync``
"""
import os
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.components.rhel_version import IsRhel7, IsRhel8, IsRhel9


@datasource(HostContext, [IsRhel7, IsRhel8, IsRhel9])
def corosync_cmapctl_cmds(broker):
    """
    corosync-cmapctl use different arguments on RHEL7 and RHEL8.

    Returns:
        list: A list of related corosync-cmapctl commands based the RHEL version.
    """
    corosync_cmd = '/usr/sbin/corosync-cmapctl'
    if os.path.exists(corosync_cmd):
        print(broker, broker.get(IsRhel7))
        # RHEL 7
        if broker.get(IsRhel7):
            return [
                corosync_cmd,
                ' '.join([corosync_cmd, '-d runtime.schedmiss.timestamp']),
                ' '.join([corosync_cmd, '-d runtime.schedmiss.delay'])]
        # Others
        elif broker.get(IsRhel8) or broker.get(IsRhel9):
            return [
                corosync_cmd,
                ' '.join([corosync_cmd, '-m stats']),
                ' '.join([corosync_cmd, '-C schedmiss'])]

    raise SkipComponent()



import pytest

from insights.specs import Specs
from insights.core import filters
from insights.specs.datasources.user_group import group_filters
from insights.core.dr import SkipComponent


def setup_function(func):
    if Specs.group_info in filters._CACHE:
        del filters._CACHE[Specs.group_info]
    if Specs.group_info in filters.FILTERS:
        del filters.FILTERS[Specs.group_info]

    if func is test_group_filters:
        filters.add_filter(Specs.group_info, ["wheel", "mem"])
    if func is test_group_filters_empty:
        filters.add_filter(Specs.group_info, [])


def test_group_filters():
    broker = {}
    result = group_filters(broker)
    assert 'mem wheel' == result


def test_group_filters_empty():
    broker = {}
    with pytest.raises(SkipComponent):
        group_filters(broker)



import pytest
import doctest

from insights.parsers import SkipException, sap_hana_python_script
from insights.parsers.sap_hana_python_script import HanaLandscape
from insights.tests import context_wrap

LANDSCAPE_SCALE_UP = """
| Host   | Host   | Host   | Failover | Remove | Storage   | Failover     | Failover     | NameServer  | NameServer  | IndexServer | IndexServer |
|        | Active | Status | Status   | Status | Partition | Config Group | Actual Group | Config Role | Actual Role | Config Role | Actual Role |
| ------ | ------ | ------ | -------- | ------ | --------- | ------------ | ------------ | ----------- | ----------- | ----------- | ----------- |
| node1  | yes    | ok     |          |        |         1 | default      | default      | master 1    | master      | worker      | master      |
overall host status: ok
""".strip()

LANDSCAPE_SCALE_OUT = """
| Host   | Host   | Host   | Failover | Remove | Storage   | Failover     | Failover     | NameServer  | NameServer  | IndexServer | IndexServer |
|        | Active | Status | Status   | Status | Partition | Config Group | Actual Group | Config Role | Actual Role | Config Role | Actual Role |
| ------ | ------ | ------ | -------- | ------ | --------- | ------------ | ------------ | ----------- | ----------- | ----------- | ----------- |
| node1  | yes    | info   |          |        | 0         | default      | default      | master 1    | slave       | worker      | standby     |
| node2  | yes    | ok     |          |        | 2         | default      | default      | master 2    | slave       | worker      | slave       |
| node3  | yes    | info   |          |        | 1         | default      | default      | master 3    | master      | standby     | master      |
overall host status: ok
""".strip()

LANDSCAPE_SCALE_UP_NG = """
nameserver vm37-39:30201 not responding.
nameserver vm37-39:30201 not responding.
| Host    | Host   | Host   | Failover | Remove | Storage   | Storage   | Failover | Failover | NameServer | NameServer | IndexServer | IndexServer | Host             | Host   | Worker  | Worker |
|         | Active | Status | Status   | Status | Config    | Actual    | Config   | Actual   | Config     | Actual     | Config      | Actual      | Config           | Actual | Config  | Actual |
|         |        |        |          |        | Partition | Partition | Group    | Group    | Role       | Role       | Role        | Role        | Roles            | Roles  | Groups  | Groups |
| ------- | ------ | ------ | -------- | ------ | --------- | --------- | -------- | -------- | ---------- | ---------- | ----------- | ----------- | ---------------- | ------ | ------- | ------ |
| vm37-39 | no     | error  | ?        | ?      | ?         | ?         | ?        | ?        | master 1   | ?          | worker      | ?           | worker xs_worker | ?      | default | ?      |

overall host status: error
""".strip()

LANDSCAPE_SCALE_UP_AB_1 = """
| Host   | Host   | Host   | Failover | Remove | Storage   | Failover     | Failover     | NameServer  | NameServer  | IndexServer | IndexServer |
|        | Active | Status | Status   | Status | Partition | Config Group | Actual Group | Config Role | Actual Role | Config Role | Actual Role |
| ------ | ------ | ------ | -------- | ------ | --------- | ------------ | ------------ | ----------- | ----------- | ----------- | ----------- |
""".strip()


def test_doc_examples():
    env = {'hana_sta': HanaLandscape(context_wrap(LANDSCAPE_SCALE_UP))}
    failed, total = doctest.testmod(sap_hana_python_script, globs=env)
    assert failed == 0


def test_HanaLandscape_ab():
    with pytest.raises(SkipException):
        HanaLandscape(context_wrap(LANDSCAPE_SCALE_UP_AB_1))


def test_HanaLandscape():
    d = HanaLandscape(context_wrap(LANDSCAPE_SCALE_UP))
    assert len(d) == 1
    assert d[0]['Host'] == 'node1'
    assert d[0]['Failover Config Group'] == 'default'
    assert d[0]['Failover Status'] == ''
    assert d.overall_status == 'ok'
    assert d.scale_up is True
    assert d.scale_out is False

    d = HanaLandscape(context_wrap(LANDSCAPE_SCALE_UP_NG))
    assert len(d) == 1
    assert d[0]['Host'] == 'vm37-39'
    assert d[0]['Failover Config Group'] == '?'
    assert d[0]['Failover Status'] == '?'
    assert d[0]['Host Status'] == 'error'
    assert d.overall_status == 'error'
    assert d.scale_up is True
    assert d.scale_out is False

    d = HanaLandscape(context_wrap(LANDSCAPE_SCALE_OUT))
    assert len(d) == 3
    assert d.overall_status == 'ok'
    assert d[0]['Host'] == 'node1'
    assert d[0]['Failover Config Group'] == 'default'
    assert d[0]['Failover Status'] == ''
    assert d[0]['Host Status'] == 'info'
    assert d[1]['Host'] == 'node2'
    assert d[1]['Failover Config Group'] == 'default'
    assert d[1]['Failover Status'] == ''
    assert d[1]['Host Status'] == 'ok'
    assert d[2]['Host'] == 'node3'
    assert d[2]['Failover Config Group'] == 'default'
    assert d[2]['Failover Status'] == ''
    assert d[2]['Host Status'] == 'info'
    assert d.scale_up is False
    assert d.scale_out is True



import pytest
import doctest
from insights.tests import context_wrap
from insights.parsers import SkipException, db2
from insights.parsers.db2 import Db2ls

DB2LS_A_C = """
#PATH:VRMF:FIXPACK:SPECIAL:INSTALLTIME:INSTALLERUID
/opt/ibm/db2/V11.5:11.5.6.0:0 ::Fri Jan 14 20:20:07 2022 CST :0
/opt/ibm/db2/V11.5_01:11.5.7.0:0 ::Fri Feb 11 10:34:51 2022 CST :0
"""

DB2LS_A_C_EMPTY1 = ""
DB2LS_A_C_EMPTY2 = "#PATH:VRMF:FIXPACK:SPECIAL:INSTALLTIME:INSTALLERUID"


def test_db2ls():
    db2ls = Db2ls(context_wrap(DB2LS_A_C))
    assert len(db2ls) == 2
    assert db2ls[0]['PATH'] == '/opt/ibm/db2/V11.5'
    assert db2ls[0]['VRMF'] == '11.5.6.0'
    assert db2ls[0]['FIXPACK'] == '0'
    assert db2ls[0]['SPECIAL'] == ''
    assert db2ls[0]['INSTALLTIME'] == 'Fri Jan 14 20:20:07 2022 CST'
    assert db2ls[0]['INSTALLERUID'] == '0'
    assert db2ls[1]['PATH'] == '/opt/ibm/db2/V11.5_01'
    assert db2ls[1]['VRMF'] == '11.5.7.0'
    assert db2ls[1]['FIXPACK'] == '0'
    assert db2ls[1]['SPECIAL'] == ''
    assert db2ls[1]['INSTALLTIME'] == 'Fri Feb 11 10:34:51 2022 CST'
    assert db2ls[1]['INSTALLERUID'] == '0'

    with pytest.raises(SkipException):
        Db2ls(context_wrap(DB2LS_A_C_EMPTY1))
    with pytest.raises(SkipException):
        Db2ls(context_wrap(DB2LS_A_C_EMPTY2))


def test_db2ls_doc_examples():
    env = {'db2ls': Db2ls(context_wrap(DB2LS_A_C))}
    failed, total = doctest.testmod(db2, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.parsers import (
    satellite_postgresql_query, ParseException, SkipException)
from insights.core.plugins import ContentException
from insights.tests import context_wrap


SATELLITE_POSTGRESQL_WRONG_1 = '''
-bash: psql: command not found
'''

SATELLITE_POSTGRESQL_WRONG_2 = '''
su: user postgres does not exist
'''

SATELLITE_POSTGRESQL_WRONG_3 = '''
psql: FATAL:  database "foreman" does not exist
'''

SATELLITE_POSTGRESQL_WRONG_4 = '''
'''

SATELLITE_POSTGRESQL_WRONG_5 = '''
name,default,value
'''.strip()

SATELLITE_SETTINGS_1 = '''
name,value,default
unregister_delete_host,"--- true
...","--- false
..."
destroy_vm_on_host_delete,,"--- true
..."
'''

SATELLITE_SETTINGS_2 = '''
name,value,default
unregister_delete_host,,"--- false
..."
destroy_vm_on_host_delete,,"--- true
..."
'''

SATELLITE_SETTINGS_3 = '''
name,value,default
unregister_delete_host,"--- false
...","--- true
..."
destroy_vm_on_host_delete,"--- false
...","--- true
..."
'''

SATELLITE_SETTINGS_WITH_DIFFERENT_TYPES = '''
name,value,default
http_proxy_except_list,,--- []
trusted_hosts,,--- []
oidc_audience,,--- []
ignored_interface_identifiers,,"---
- lo
- en*v*
- usb*
- vnet*
- macvtap*
- _vdsmdummy_
- veth*
- docker*
- tap*
- qbr*
- qvb*
- qvo*
- qr-*
- qg-*
- vlinuxbr*
- vovsbr*"
dns_timeout,,"---
- 5
- 10
- 15
- 20"
foreman_tasks_troubleshooting_url,,"--- https://access.redhat.com/solutions/satellite6-tasks#%{label}
..."
remote_execution_ssh_user,,"--- root
..."
foreman_tasks_sync_task_timeout,,"--- 120
..."
foreman_tasks_proxy_action_retry_count,,"--- 4
..."
'''

SATELLITE_SETTINGS_BAD_1 = '''
name,value
unregister_delete_host,"--- true
..."
destroy_vm_on_host_delete,
'''

SATELLITE_SETTINGS_BAD_2 = '''
name,value,default
unregister_delete_host,"--- true:: def
...","--- false
..."
destroy_vm_on_host_delete,,"--- true
..."
'''

SATELLITE_COMPUTE_RESOURCE_1 = '''
name,type
test_compute_resource1,Foreman::Model::Libvirt
test_compute_resource2,Foreman::Model::RHV
'''

SATELLITE_SCA_INFO_1 = '''
displayname,content_access_mode
Default Organization,entitlement
Orgq,org_environment
'''

SATELLITE_SCA_INFO_2 = '''
displayname,content_access_mode
Default Organization,entitlement
Orgq,entitlement
'''

SATELLITE_KATELLO_ROOT_REPOSITORIES = """
id,name
54,testa
55,testb
"""

SATELLITE_TASK_RESERVERDRESOURCE_CONTENT = """
count
0
"""

SATELLITE_QUERY_DATA1 = """
logname: no login name
/etc/profile.d/hkuser.sh: line 40: HISTFILE: readonly variable
id,name
1,Puppet_Base
""".strip()

SATELLITE_QUERY_DATA2 = """
logname: no login name
/etc/profile.d/hkuser.sh: line 40: HISTFILE: readonly variable
id,name
""".strip()

SATELLITE_QUERY_DATA3 = """
logname: no login name
/etc/profile.d/hkuser.sh: line 40: HISTFILE: readonly variable
""".strip()

SATELLITE_CAPSULES_WITH_BACKGROUND_DOWNLOADPOLICY = """
name
capsule1.test.com
capsule2.test.com
"""

SATELLITE_CAPSULES_WITH_BACKGROUND_DOWNLOADPOLICY_2 = """
name
Last login: Sun May  8 01:51:06 EDT 2022
"""

SATELLITE_REPOS_INFO = """
id,name,url,download_policy
2,Red Hat Satellite Tools 6.8 for RHEL 7 Server RPMs x86_64,,on_demand
3,Red Hat Enterprise Linux 8 for x86_64 - AppStream RPMs 8,https://xx.xx.com/rhel8/8/x86_64/appstream/os,background
4,Red Hat Enterprise Linux 7 Server RPMs x86_64 7Server,https://xx.xx.com/server/7/7Server/x86_64/os,background
"""

SATELLITE_KATELLO_REPOS_WITH_MULTI_REF = """
repository_href,count
/pulp/api/v3/repositories/rpm/rpm/64e1ddf8-025e-45f2-b2f0-04b874674671/,3
/pulp/api/v3/repositories/rpm/rpm/sfwrsrw45sfse-45f2-b2f0-04b874675688/,2
""".strip()


def test_HTL_doc_examples():
    settings = satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_1))
    resources_table = satellite_postgresql_query.SatelliteComputeResources(context_wrap(SATELLITE_COMPUTE_RESOURCE_1))
    sat_sca_info = satellite_postgresql_query.SatelliteSCAStatus(context_wrap(SATELLITE_SCA_INFO_1))
    repositories = satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories(context_wrap(SATELLITE_KATELLO_ROOT_REPOSITORIES))
    tasks = satellite_postgresql_query.SatelliteCoreTaskReservedResourceCount(context_wrap(SATELLITE_TASK_RESERVERDRESOURCE_CONTENT))
    capsules = satellite_postgresql_query.SatelliteQualifiedCapsules(context_wrap(SATELLITE_CAPSULES_WITH_BACKGROUND_DOWNLOADPOLICY))
    repos = satellite_postgresql_query.SatelliteQualifiedKatelloRepos(context_wrap(SATELLITE_REPOS_INFO))
    multi_ref_katello_repos = satellite_postgresql_query.SatelliteKatellloReposWithMultipleRef(context_wrap(SATELLITE_KATELLO_REPOS_WITH_MULTI_REF))
    globs = {
        'table': settings,
        'resources_table': resources_table,
        'sat_sca_info': sat_sca_info,
        'katello_root_repositories': repositories,
        'tasks': tasks,
        'capsules': capsules,
        'repos': repos,
        'multi_ref_katello_repos': multi_ref_katello_repos
    }
    failed, _ = doctest.testmod(satellite_postgresql_query, globs=globs)
    assert failed == 0


def test_no_headers():
    with pytest.raises(NotImplementedError):
        satellite_postgresql_query.SatellitePostgreSQLQuery(context_wrap(SATELLITE_POSTGRESQL_WRONG_4))


def test_basic_output_with_satellite_admin_setting():
    with pytest.raises(ContentException):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_POSTGRESQL_WRONG_1))
    with pytest.raises(ValueError):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_POSTGRESQL_WRONG_2))
    with pytest.raises(ValueError):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_POSTGRESQL_WRONG_3))
    with pytest.raises(ValueError):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_POSTGRESQL_WRONG_4))
    with pytest.raises(SkipException):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_POSTGRESQL_WRONG_5))


def test_satellite_admin_settings():
    settings = satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_2))
    assert (len(settings)) == 2
    assert not settings.get_setting('unregister_delete_host')
    assert settings.get_setting('destroy_vm_on_host_delete')

    settings = satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_3))
    assert (len(settings)) == 2
    assert not settings.get_setting('unregister_delete_host')
    assert not settings.get_setting('destroy_vm_on_host_delete')
    assert settings.get_setting('non_exist_column') is None

    table = satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_WITH_DIFFERENT_TYPES))
    setting_value = table.get_setting('ignored_interface_identifiers')
    assert isinstance(setting_value, list)
    for item in ['lo', 'en*v*', 'usb*', 'vnet*', 'macvtap*', '_vdsmdummy_', 'veth*',
                 'docker*', 'tap*', 'qbr*', 'qvb*', 'qvo*', 'qr-*', 'qg-*',
                 'vlinuxbr*', 'vovsbr*']:
        assert item in setting_value
    setting_value = table.get_setting('foreman_tasks_troubleshooting_url')
    assert isinstance(setting_value, str)
    assert setting_value == 'https://access.redhat.com/solutions/satellite6-tasks#%{label}'
    setting_value = table.get_setting('foreman_tasks_sync_task_timeout')
    assert isinstance(setting_value, int)
    assert setting_value == 120


def test_satellite_admin_settings_exception():
    with pytest.raises(ValueError):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_BAD_1))
    with pytest.raises(ParseException):
        satellite_postgresql_query.SatelliteAdminSettings(context_wrap(SATELLITE_SETTINGS_BAD_2))


def test_satellite_compute_resources():
    resources_table = satellite_postgresql_query.SatelliteComputeResources(context_wrap(SATELLITE_COMPUTE_RESOURCE_1))
    assert len(resources_table) == 2
    rows = resources_table.search(type='Foreman::Model::RHV')
    assert len(rows) == 1
    assert rows[0]['name'] == 'test_compute_resource2'


def test_satellite_sca():
    sat_sca_info = satellite_postgresql_query.SatelliteSCAStatus(context_wrap(SATELLITE_SCA_INFO_2))
    assert not sat_sca_info.sca_enabled


def test_satellite_katello_empty_url_repositories():
    repositories = satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories(context_wrap(SATELLITE_KATELLO_ROOT_REPOSITORIES))
    assert repositories[1]['name'] == 'testb'

    table = satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories(context_wrap(SATELLITE_QUERY_DATA1))
    assert len(table) == 1
    assert table[0]['id'] == '1'
    assert table[0]['name'] == 'Puppet_Base'

    with pytest.raises(SkipException):
        satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories(context_wrap(SATELLITE_QUERY_DATA2))

    with pytest.raises(ValueError):
        satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories(context_wrap(SATELLITE_QUERY_DATA3))


def test_satellite_taskreservedresource():
    tasks = satellite_postgresql_query.SatelliteCoreTaskReservedResourceCount(context_wrap(SATELLITE_TASK_RESERVERDRESOURCE_CONTENT))
    assert tasks[0]['count'] == '0'


def test_satellite_qulified_capsules():
    capsules = satellite_postgresql_query.SatelliteQualifiedCapsules(context_wrap(SATELLITE_CAPSULES_WITH_BACKGROUND_DOWNLOADPOLICY))
    assert len(capsules) == 2
    assert capsules[0]['name'] == 'capsule1.test.com'

    with pytest.raises(SkipException):
        satellite_postgresql_query.SatelliteQualifiedCapsules(context_wrap(SATELLITE_CAPSULES_WITH_BACKGROUND_DOWNLOADPOLICY_2))


def test_satellite_qulified_repos():
    repos = satellite_postgresql_query.SatelliteQualifiedKatelloRepos(context_wrap(SATELLITE_REPOS_INFO))
    assert len(repos) == 3
    assert repos[0]['name'] == 'Red Hat Satellite Tools 6.8 for RHEL 7 Server RPMs x86_64'
    background_repos = repos.search(download_policy='background')
    assert len(background_repos) == 2
    assert background_repos[0]['name'] == 'Red Hat Enterprise Linux 8 for x86_64 - AppStream RPMs 8'


def test_satellite_repos_with_multiptle_ref():
    repos = satellite_postgresql_query.SatelliteKatellloReposWithMultipleRef(context_wrap(SATELLITE_KATELLO_REPOS_WITH_MULTI_REF))
    assert len(repos) == 2
    assert repos[0]['repository_href'] == '/pulp/api/v3/repositories/rpm/rpm/64e1ddf8-025e-45f2-b2f0-04b874674671/'



"""
CloudCfg - datasource ``cloud_cfg``
===================================
"""
from insights import JSONParser, parser
from insights.specs import Specs


@parser(Specs.cloud_cfg)
class CloudCfg(JSONParser):
    """This parser parses the output of ``cloud_cfg`` datasource.

    Typical output from the datasource is::

        {"ssh_deletekeys": 1, "network": {"version": 1, "config": [{"type": "physical", "name": "eth0", "subnets": [{"type": "dhcp"}, {"type": "dhcp6"}]}]}, "debug": {"output": "/var/log/cloud-init-debug.log", "verbose": true}}

    Attributes:

        data(dict): Cloud-init network configuration.

    Examples:
        >>> cloud_cfg.data['network']['version'] == 1
        True
        >>> cloud_cfg.data['network']['config'] == [{"type": "physical", "name": "eth0", "subnets": [{"type": "dhcp"}, {"type": "dhcp6"}]}]
        True
"""
    pass



from insights.parsers import ParseException
from insights.parsers.ipcs import IpcsM, IpcsMP
from insights.combiners import ipcs_shared_memory
from insights.combiners.ipcs_shared_memory import IpcsSharedMemory
from insights.tests import context_wrap
import doctest
import pytest

IPCS_M = """
------ Shared Memory Segments --------
key        shmid      owner      perms      bytes      nattch     status
0x0052e2c1 0          postgres   600        37879808   26
0x0052e2c2 1          postgres   600        41222144   24
""".strip()

IPCS_M_P = """
------ Shared Memory Creator/Last-op --------
shmid      owner      cpid       lpid
0          postgres   1833       23566
1          postgres   1105       9882
""".strip()

IPCS_M_P_AB = """
------ Shared Memory Creator/Last-op --------
shmid      owner      cpid       lpid
0          postgres   1833       23566
""".strip()


def test_ipcs_shm():
    shm = IpcsM(context_wrap(IPCS_M))
    shmp = IpcsMP(context_wrap(IPCS_M_P))
    rst = IpcsSharedMemory(shm, shmp)
    assert rst.get_shm_size_of_pid('1833') == 37879808
    assert rst.get_shm_size_of_pid('33') == 0


def test_ipcs_shm_abnormal():
    shm = IpcsM(context_wrap(IPCS_M))
    shmp = IpcsMP(context_wrap(IPCS_M_P_AB))
    with pytest.raises(ParseException) as pe:
        IpcsSharedMemory(shm, shmp)
    assert "The output of 'ipcs -m' doesn't match with 'ipcs -m -p'." in str(pe)


def test_doc_examples():
    env = {
            'ism': IpcsSharedMemory(
                        IpcsM(context_wrap(IPCS_M)),
                        IpcsMP(context_wrap(IPCS_M_P)))
          }
    failed, total = doctest.testmod(ipcs_shared_memory, globs=env)
    assert failed == 0



from insights.parsers import lsmod
from insights.tests import context_wrap

LSMODINFO = """
Module                  Size  Used by
xt_CHECKSUM            12549  1
ipt_MASQUERADE         12678  3
nf_nat_masquerade_ipv4    13412  1 ipt_MASQUERADE
tun                    27141  3
ip6t_rpfilter          12546  1
ip6t_REJECT            12939  2
ipt_REJECT             12541  4
xt_conntrack           12760  12
ebtable_nat            12807  0
ebtable_broute         12731  0
bridge                119560  1 ebtable_broute
stp                    12976  1 bridge
llc                    14552  2 stp,bridge
ebtable_filter         12827  0
ebtables               30913  3 ebtable_broute,ebtable_nat,ebtable_filter
ip6table_nat           12864  1
""".strip()


def test_get_modules_info():
    mod_dict = lsmod.LsMod(context_wrap(LSMODINFO))
    assert len(mod_dict.data) == 16
    assert 'xt_CHECKSUM' in mod_dict
    assert mod_dict['tun'].get('depnum') == '3'
    assert mod_dict['llc'].get('deplist') == 'stp,bridge'
    assert mod_dict['ip6table_nat'].get('size') == '12864'



"""
Custom datasource for package_provides
"""
import logging
import signal

from insights.combiners.ps import Ps
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.specs import Specs

from . import get_running_commands, DEFAULT_SHELL_TIMEOUT

logger = logging.getLogger(__name__)


def get_package(ctx, file_path):
    """
    Get the RPM package that owns the specified filename with path

    Arguments:
        ctx: The current execution context
        file_path(str): The full path and filename for RPM query

    Returns:
        str: The name of the RPM package that provides the ``file``
        or None if file is not associated with an RPM.
    """
    rc, resolved = ctx.shell_out(
        "/usr/bin/readlink -e {0}".format(file_path),
        timeout=DEFAULT_SHELL_TIMEOUT,
        keep_rc=True
    )
    if rc == 0 and resolved:
        rc, pkg = ctx.shell_out(
            "/usr/bin/rpm -qf {0}".format(resolved[0]),
            timeout=DEFAULT_SHELL_TIMEOUT,
            keep_rc=True,
            signum=signal.SIGTERM
        )
        if rc == 0 and pkg:
            return pkg[0]


@datasource(Ps, HostContext)
def cmd_and_pkg(broker):
    """
    Collect a list of running commands and the associated RPM package providing those commands.
    The commands are based on filters so rules must add the desired commands as filters to
    enable collection.  If a command is not provided by an RPM then it will not be included
    in the output.

    In order for the full command line to be present in the Ps combiner a filter must be added
    to the spec ``ps_auxww``.  A filter must also be added to ``package_provides_command`` so
    this datasource will look for the command in Ps.

    Arguments:
        broker: the broker object for the current session

    Returns:
        DatasourceProvider: Returns the collected information as a file with 1 line per command

    Raises:
        SkipComponent: Raised if no data is collected
    """
    commands = get_filters(Specs.package_provides_command)
    """ list: List of commands to search for, added as filters for the spec """

    if commands:
        pkg_cmd = list()
        for cmd in get_running_commands(broker[Ps], broker[HostContext], list(commands)):
            pkg = get_package(broker[HostContext], cmd)
            if pkg is not None:
                pkg_cmd.append("{0} {1}".format(cmd, pkg))
        if pkg_cmd:
            return DatasourceProvider('\n'.join(pkg_cmd), relative_path='insights_commands/package_provides_command')
    raise SkipComponent



"""
QpiddConfig - file ``/etc/qpid/qpidd.conf``
===========================================
"""
from insights.specs import Specs

from . import split_kv_pairs
from .. import LegacyItemAccess, Parser, get_active_lines, parser


@parser(Specs.qpidd_conf)
class QpiddConf(Parser, LegacyItemAccess):
    """
    Parse the qpidd configuration file.

    Produces a simple dictionary of keys and values from the configuration
    file contents , stored in the ``data`` attribute.  The object also
    functions as a dictionary itself thanks to the
    :py:class:`insights.core.LegacyItemAccess` mixin class.

    Sample configuration file::

        # Configuration file for qpidd. Entries are of the form:
        # name=value
        #
        # (Note: no spaces on either side of '='). Using default settings:
        # "qpidd --help" or "man qpidd" for more details.
        #cluster-mechanism=ANONYMOUS
        log-enable=error+
        log-to-syslog=yes
        auth=no
        require-encryption=yes
        ssl-require-client-authentication=yes
        ssl-port=5672
        ssl-cert-db=/etc/pki/katello/nssdb
        ssl-cert-password-file=/etc/pki/katello/nssdb/nss_db_password-file
        ssl-cert-name=broker

        interface=lo

    Examples:
        >>> qpidd_conf['auth']
        'no'
        >>> 'require-encryption' in qpidd_conf
        True
    """

    def parse_content(self, content):
        self.data = split_kv_pairs(get_active_lines(content))
